/*******************************************************************************
*
*  Web Vengeance Poo Pack - diaper.c
*
*  The Poo Pack's Diaper Launcher effect routines.
*
*	Copyright (c) 1997 SegaSoft, Inc.  All Rights Reserved.
*
*------------------------------------------------------------------------------
*                   CONFIDENTIAL AND PROPRIETARY INFORMATION
*------------------------------------------------------------------------------
*  The contents of this file are the property of SegaSoft, Inc., 
*  and may not be used or duplicated without the permission of SegaSoft, Inc.
*------------------------------------------------------------------------------
*
*  Edit History:
*		97-Oct-29	DJS	Creation.
*
*  Initials:
*  	DJS	Dan J. Su
*
*******************************************************************************/
#include <ddraw.h>
#include <windows.h>
#include <stdlib.h>
#include "BlasterEffects.h"
#include "EffectType.h"

#define DEFINE_DIAPER
#include "diaper.h"
#include "main.h"


// ---------- Defines
#define MAX_OBJECTS_DIAPER       ( 5UL )  // Can only throw this many diapers at a time

   // Gravity ( acceleration downward ) for each diaper, in pixels per vblank per vblank
   // Each vblank is approx. 1/30th of a second
#define G_ACCELERATION           (  4 )
#define PI                       ( 3.1415926535f )
#define LAUNCHER_Y               ( nScreenHeight - s_aTextures[ TEXTURE_LAUNCHER_LOW_CLOSED ].height )
#define SMOKE_Y                  ( LAUNCHER_Y - s_aTextures[ TEXTURE_LAUNCHER_SMOKE_0 ].height + 335 )
#define DIAPER_STARTING_Y        ( LAUNCHER_Y - s_aTextures[ TEXTURE_DIAPER_SPIN_A_0 ].height + 268 )
#define DIAPER_HOT_SPOT_Y        ( 170 )  // y-offset to center of diaper from bottom of splat animation frame
#define NUM_OF_FRAME_DELAYS      (  5 )   // Number of different frame delay values
#define DEFAULT_ANIM_FRAME_DELAY (  2 )   // Draw a new frame every ANIM_FRAME_DELAY vblanks
                                          // Note that each diaper uses it's own frame delay,
                                          // however, splat animations use this predefined delay
#define ANIM_FRAME_DELAY_SMOKE   (  3 )   // Smoke blows out slower than diaper and diaper launcher's animations

   // ----- Texture indexes
#define TEXTURE_LAUNCHER_LOW_CLOSED       (  0 )   // Diaper launcher aimed low, lid is closed
#define TEXTURE_LAUNCHER_LOW_OPEN         (  1 )   // Diaper launcher aimed low, lid is half open
#define TEXTURE_LAUNCHER_LOW_OPEN_WIDE    (  2 )   // Diaper launcher aimed low, lid is wide open
#define TEXTURE_LAUNCHER_MED_CLOSED       (  3 )
#define TEXTURE_LAUNCHER_MED_OPEN         (  4 )
#define TEXTURE_LAUNCHER_MED_OPEN_WIDE    (  5 )
#define TEXTURE_LAUNCHER_HIGH_CLOSED      (  6 )
#define TEXTURE_LAUNCHER_HIGH_OPEN        (  7 )
#define TEXTURE_LAUNCHER_HIGH_OPEN_WIDE   (  8 )
#define TEXTURE_LAUNCHER_SMOKE_0          (  9 )   // Smoke coming out of the launcher
#define TEXTURE_LAUNCHER_SMOKE_1          ( 10 )
#define TEXTURE_LAUNCHER_SMOKE_2          ( 11 )
#define TEXTURE_LAUNCHER_SMOKE_3          ( 12 )
#define TEXTURE_DIAPER_SPIN_A_0           ( 13 )   // Two different spinning animations
#define TEXTURE_DIAPER_SPIN_A_1           ( 14 )   // as the diaper travels through the air
#define TEXTURE_DIAPER_SPIN_A_2           ( 15 )
#define TEXTURE_DIAPER_SPIN_A_3           ( 16 )
#define TEXTURE_DIAPER_SPIN_A_4           ( 17 )
#define TEXTURE_DIAPER_SPIN_B_0           ( 18 )
#define TEXTURE_DIAPER_SPIN_B_1           ( 19 )
#define TEXTURE_DIAPER_SPIN_B_2           ( 20 )
#define TEXTURE_DIAPER_SPIN_B_3           ( 21 )
#define TEXTURE_DIAPER_SPIN_B_4           ( 22 )
#define TEXTURE_DIAPER_SPIN_B_5           ( 23 )
#define TEXTURE_SPLAT_A_0                 ( 24 )   // Two different splat sequences
#define TEXTURE_SPLAT_A_1                 ( 25 )
#define TEXTURE_SPLAT_A_2                 ( 26 )
#define TEXTURE_SPLAT_A_3                 ( 27 )
#define TEXTURE_SPLAT_A_4                 ( 28 )
#define TEXTURE_SPLAT_A_5                 ( 29 )   // Final diaper splat texture
#define TEXTURE_SPLAT_B_0                 ( 30 )
#define TEXTURE_SPLAT_B_1                 ( 31 )
#define TEXTURE_SPLAT_B_2                 ( 32 )
#define TEXTURE_SPLAT_B_3                 ( 33 )
#define TEXTURE_SPLAT_B_4                 ( 34 )
#define TEXTURE_SPLAT_B_5                 ( 35 )   // Final diaper splat texture
#define TEXTURE_DEFENSE                   ( 36 )   // Umbrella defense
#define TEXTURE_DEFENSE_DAMAGE_0          ( 37 )   // Diaper stains on umbrella
#define TEXTURE_DEFENSE_DAMAGE_1          ( 38 )
#define TEXTURE_DEFENSE_DAMAGE_2          ( 39 )
#define NUM_TEXTURES_TOTAL                ( 40 )

   // ----- Sound indexes
#define SOUND_NO_AMMO_0          ( 0 )
#define SOUND_NO_AMMO_1          ( 1 )
#define SOUND_LAUNCH             ( 2 )    // Sound when launching a diaper
#define SOUND_HIT_PAGE           ( 3 )    // Sound of diaper hitting the page
#define SOUND_BLOCKED            ( 4 )    // Diaper blocked by umbrella
#define NUM_SOUNDS_TOTAL         ( 5 )

   // ----- Defense info
#define DEFENSE_STATE_SEEKING    ( 1 )    // Defense mechanism needs to find a new target, cursor or diaper
#define DEFENSE_STATE_MOVING     ( 2 )    // Defense mechanism is moving towards target
#define DEFENSE_FOLLOW_CURSOR    ( -999 ) // Pick an invalid x-coordinate to indicate
                                          // there are no diapers to target
#define DEFENSE_MIN_STEP         ( 2 )    // Minimum # steps to reach destination
#define DEFENSE_MAX_STEP         ( 5 )    // Maximum # of steps to reach destination

   // Collision box offsets, relative to top-left corner of the TEXTURE_DIAPER_SPIN_A_4 bitmap
   // This area should be relatively small and centered where the diaper would hit the page
#define DIAPER_COLLISION_BOX_X1     (  10 )
#define DIAPER_COLLISION_BOX_X2     ( 122 )
#define DIAPER_COLLISION_BOX_Y1     (  20 )
#define DIAPER_COLLISION_BOX_Y2     ( 110 )

   // ----- States for the diaper launching sequence
#define DIAPER_STATE_INVALID        ( 0 )    // Marks an unused diaper object
#define DIAPER_STATE_LAUNCHING      ( 1 )    // Diaper is being launched
#define DIAPER_STATE_SPINNING       ( 2 )    // Diaper has been thrown in the air
#define DIAPER_STATE_SPLATTERING    ( 3 )    // Diaper is splattering on the page
#define DIAPER_STATE_HIT_PAGE       ( 4 )    // Diaper is about to leave a mark on the web page
#define DIAPER_STATE_BLOCKED        ( 5 )    // Diaper has been blocked


// ---------- Structures and Types
   // Each diaper object simply contains the FXOBJECT for the diaper, 
   // which is used for the flying diaper and the final diaper image on the page
typedef struct diaper_s
{
   DWORD       dwState;    // Current state we're in.  See DIAPER_STATE_... defines above.
   FXOBJECT *  pfxDiaper;  // Pointer to FXOBJECT in main app's global Projectile[] array
                           // This is the hole, crater, etc. that gets saved for the current web page.
                           // dwFlags member is used to track the final x position
                           // dwFlags2 member is used to track the final y position
                           // fScaleY contains object's y-position as a float
                           // fAngle member is used to store y-velocity as a float
   WORD        wFrameDelay;   // Number of vblanks in between animation frames
                              // The lower the target, the shorter the delay and hence
                              // the sooner the diaper will hit the target.
                              // This helps us shape the parabolic trajectory of the diaper so it looks nice :)
   WORD        wPad;
} diaper_t;


// ---------- Local Functions
static void DiaperCheckInputDevices          ( void );
static void DiaperCheckCheatCodes            ( void );
static void DiaperStart                      ( void );
static void DiaperCreateSplatter             ( int x, int y );
static void DiaperCalculate                  ( void );
static void DiaperDrawBuffer                 ( DDRAW_DATA *pDDrawData );
static void DiaperEraseBuffer                ( DDRAW_DATA *pDDrawData );
static void DiaperDrawScreen                 ( DDRAW_DATA *pDDrawData );
static void DiaperDefenseMove                ( void );
static void DiaperDefenseCheckForCollision   ( void );


// ---------- Local Variables
static diaper_t   s_aDiaper[ MAX_OBJECTS_DIAPER ];       // Array of diaper objects
static FXOBJECT   s_fxLauncher;                          // Diaper launcher
                                                         // nFlags member is used to indicate anim playback direction
                                                         //    1  = play forwards, open the lid
                                                         //   -1  = play backwards, close the lid
                                                         //    0  = not playing any animation, lid is closed
                                                         // dwFlags member points to the diaper_t object to launch
                                                         //   when the launcher is done opening its lid.
                                                         // dwFlags2 member is an index into s_awLauncherRecoilDistances[]
static FXOBJECT   s_fxSmoke;                             // Smoke coming out of the diaper launcher
                                                         // nFrameTime used to time animation
                                                         // nFrame contains current texture
                                                         // nFlags member is set to 1 if we're to draw the smoke,
                                                         //   = ERASE if we need to erase the last smoke anim frame from the screen
                                                         //   = 0 if we don't want to draw smoke
static TEXTURE    s_aTextures[ NUM_TEXTURES_TOTAL ];     // Textures
static int        s_aiSounds[ NUM_SOUNDS_TOTAL ];        // Sound indexes
static WORD       s_awFrameDelays[ NUM_OF_FRAME_DELAYS + 1 ] = 
                  { 4, 4, 3, 3, 2, 1 };
static WORD       s_awLauncherRecoilDistances[ 3 ] = { 80, 60, 40 };  // HIGH, MED, LOW, must be even


/******************************************************************************
*
*  NAME
*     DiaperUpdate
*
*  SYNOPSIS
*     iSuccess = DiaperUpdate( pDDrawData, iEffect, iMsg, iObjectCount );
*
*  DESCRIPTION
*     Starts or ends the Diaper effect.
*
*  INPUTS
*     DDRAW_DATA *   pDDrawData     = Info on current DirectDraw device
*     int            iEffect        = Effect ID, should be the ID for the diaper effect
*     int            iMsg           = Message to process i.e. start or end effect
*     int            iObjectCount   = Not used
*
*  OUTPUTS
*		int iSuccess = TRUE if update was successful, FALSE otherwise
*
*  BUGS
*		None
*
*  SEE ALSO
*     InitEffects()
*
********************************************************************************/
BOOL DiaperUpdate( DDRAW_DATA *pDDrawData, int iEffect, int iMsg, int iObjectCount )
{
   if ( FindEffect( EFFECT_NAME_DIAPER ) != iEffect )
      return ( FALSE );

   switch( iMsg )
   {
      case EFFECT_STARTED:
      {
         srand( GetTickCount() );

         // Load sounds
			s_aiSounds[ SOUND_NO_AMMO_0 ] = LoadWave( ".\\Sounds\\Poo\\Diaper\\NoAmmo1.wav" );
			s_aiSounds[ SOUND_NO_AMMO_1 ] = LoadWave( ".\\Sounds\\Poo\\Diaper\\NoAmmo2.wav" );
			s_aiSounds[ SOUND_LAUNCH ]    = LoadWave( ".\\Sounds\\Poo\\Diaper\\Launch.wav" );
         s_aiSounds[ SOUND_HIT_PAGE ]  = LoadWave( ".\\Sounds\\Poo\\Diaper\\Plop.wav" );
			s_aiSounds[ SOUND_BLOCKED ]   = LoadWave( ".\\Sounds\\Poo\\Diaper\\Blocked.wav" );

         // Load textures
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7olposa01.pcx", &s_aTextures[ TEXTURE_LAUNCHER_LOW_CLOSED ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7olposa02.pcx", &s_aTextures[ TEXTURE_LAUNCHER_LOW_OPEN ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7olposa03.pcx", &s_aTextures[ TEXTURE_LAUNCHER_LOW_OPEN_WIDE ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7olposc01.pcx", &s_aTextures[ TEXTURE_LAUNCHER_MED_CLOSED ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7olposc02.pcx", &s_aTextures[ TEXTURE_LAUNCHER_MED_OPEN ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7olposc03.pcx", &s_aTextures[ TEXTURE_LAUNCHER_MED_OPEN_WIDE ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7olpose01.pcx", &s_aTextures[ TEXTURE_LAUNCHER_HIGH_CLOSED ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7olpose02.pcx", &s_aTextures[ TEXTURE_LAUNCHER_HIGH_OPEN ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7olpose03.pcx", &s_aTextures[ TEXTURE_LAUNCHER_HIGH_OPEN_WIDE ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\Smoke01.pcx",   &s_aTextures[ TEXTURE_LAUNCHER_SMOKE_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\Smoke02.pcx",   &s_aTextures[ TEXTURE_LAUNCHER_SMOKE_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\Smoke03.pcx",   &s_aTextures[ TEXTURE_LAUNCHER_SMOKE_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\Smoke04.pcx",   &s_aTextures[ TEXTURE_LAUNCHER_SMOKE_3 ] );
         
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7opaka01.pcx",  &s_aTextures[ TEXTURE_DIAPER_SPIN_A_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7opaka02.pcx",  &s_aTextures[ TEXTURE_DIAPER_SPIN_A_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7opaka03.pcx",  &s_aTextures[ TEXTURE_DIAPER_SPIN_A_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7opaka04.pcx",  &s_aTextures[ TEXTURE_DIAPER_SPIN_A_3 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7opaka05.pcx",  &s_aTextures[ TEXTURE_DIAPER_SPIN_A_4 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7opakb01.pcx",  &s_aTextures[ TEXTURE_DIAPER_SPIN_B_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7opakb02.pcx",  &s_aTextures[ TEXTURE_DIAPER_SPIN_B_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7opakb03.pcx",  &s_aTextures[ TEXTURE_DIAPER_SPIN_B_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7opakb04.pcx",  &s_aTextures[ TEXTURE_DIAPER_SPIN_B_3 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7opakb05.pcx",  &s_aTextures[ TEXTURE_DIAPER_SPIN_B_4 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7opakb06.pcx",  &s_aTextures[ TEXTURE_DIAPER_SPIN_B_5 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatb01.pcx", &s_aTextures[ TEXTURE_SPLAT_A_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatb02.pcx", &s_aTextures[ TEXTURE_SPLAT_A_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatb03.pcx", &s_aTextures[ TEXTURE_SPLAT_A_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatb04.pcx", &s_aTextures[ TEXTURE_SPLAT_A_3 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatb05.pcx", &s_aTextures[ TEXTURE_SPLAT_A_4 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatb06.pcx", &s_aTextures[ TEXTURE_SPLAT_A_5 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatc01.pcx", &s_aTextures[ TEXTURE_SPLAT_B_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatc02.pcx", &s_aTextures[ TEXTURE_SPLAT_B_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatc03.pcx", &s_aTextures[ TEXTURE_SPLAT_B_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatc04.pcx", &s_aTextures[ TEXTURE_SPLAT_B_3 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatc05.pcx", &s_aTextures[ TEXTURE_SPLAT_B_4 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatc06.pcx", &s_aTextures[ TEXTURE_SPLAT_B_5 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Umbrella\\7dumb01.pcx",  &s_aTextures[ TEXTURE_DEFENSE ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Umbrella\\7escuf01.pcx",  &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Umbrella\\7escuf02.pcx",  &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Umbrella\\7escuf03.pcx",  &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_2 ] );

         InitObject( FX_ID_DEFENSE, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ],
                     ( nScreenWidth >> 1 ), ( nScreenHeight >> 1 ) );
         g_defense.fxObject.nFlags  = DEFENSE_STATE_SEEKING;   // Reset state flag
         g_defense.fxObject.dwFlags = 0;                       // Clear erase flag

         InitObject( -3, &s_fxLauncher, &s_aTextures[ TEXTURE_LAUNCHER_LOW_CLOSED ],
                     nMouseX - ( s_aTextures[ TEXTURE_LAUNCHER_LOW_CLOSED ].width >> 1 ),
                     LAUNCHER_Y );
         s_fxLauncher.nFlags     = 0;           // Launcher's lid is closed and not animating
         s_fxLauncher.dwFlags    = 0;           // Pointer to diaper_t object to launch
         s_fxLauncher.dwFlags2   = 0;           // Index into s_awLauncherRecoilDistances[]
         s_fxLauncher.nFrame     = TEXTURE_LAUNCHER_LOW_CLOSED;
         s_fxLauncher.nFrameTime = 0;

         InitObject( -4, &s_fxSmoke, &s_aTextures[ TEXTURE_LAUNCHER_SMOKE_0 ],
                     nMouseX - ( s_aTextures[ TEXTURE_LAUNCHER_SMOKE_0 ].width >> 1 ),
                     SMOKE_Y );
         s_fxSmoke.nFlags     = 0;
         s_fxSmoke.nFrameTime = 0;
         s_fxSmoke.nFrame     = TEXTURE_LAUNCHER_SMOKE_0;

         // Init table of diaper objects
         memset( s_aDiaper, 0, sizeof( diaper_t ) * MAX_OBJECTS_DIAPER );

         break;
      }

      case EFFECT_ENDED:
      {
         diaper_t *pDiaper, *pDiaperEnd;
         int      i;


         DeleteWaves();       // Free sounds

         for ( i = 0; i < NUM_TEXTURES_TOTAL; i++ )
            DeleteTexture( &s_aTextures[ i ] );

         // Delete all objects that are in the middle of an animation sequence.
         // They don't count as hits on the page.
         pDiaper     = &s_aDiaper[ 0 ];
         pDiaperEnd  = &s_aDiaper[ MAX_OBJECTS_DIAPER ];
         while ( pDiaper < pDiaperEnd )
         {
            if ( pDiaper->dwState )
            {
               pDiaper->dwState     = DIAPER_STATE_INVALID;
               pDiaper->pfxDiaper   = NULL;
               DeleteLastFXObject();      // Remove record of this object from the global
                                          // Projectile[] array so it doesn't get saved.
                                          // This takes care of exiting in the middle of an animation sequence
            }

            pDiaper++;
         }
         break;
      }

      default:
         break;      // Future commands do not generate an error
   }

   return ( TRUE );
}


/******************************************************************************
*
*  NAME
*     DiaperDraw
*
*  SYNOPSIS
*     iSuccess = DiaperDraw( pDDrawData, iMsg );
*
*  DESCRIPTION
*     Draws the current frame for the Diaper Launcher effect
*
*  INPUTS
*     DDRAW_DATA *   pDDrawData     = Info on current DirectDraw device
*     int            iMsg           = Message to process
*
*  OUTPUTS
*		int iSuccess = TRUE
*
*  BUGS
*		None
*
*  SEE ALSO
*     InitEffects()
*
********************************************************************************/
BOOL DiaperDraw( DDRAW_DATA *pDDrawData, int iMsg )
{
   DiaperCheckInputDevices();

   // Grab defense status from main app variable
   g_defense.byObjectAlive = nEffectsDefended;

   switch( iMsg )
   {
      case DRAW_EFFECT:
         DiaperCalculate();
         DiaperDrawBuffer( pDDrawData );      // Main drawing pipeline
         DiaperDrawScreen( pDDrawData );
         DiaperEraseBuffer( pDDrawData );
         break;

      case CALCULATE_AND_SKIP_FRAME:
         DiaperCalculate();
         break;

      default:
         break;   // Simply ignore future messages
   }

   return ( TRUE );
}


/******************************************************************************
*
*  NAME
*     DiaperCheckInputDevices
*
*  SYNOPSIS
*     DiaperCheckInputDevices()
*
*  DESCRIPTION
*     Checks for input from the mouse or keyboard.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     DiaperDraw(), DiaperCheckCheatCodes()
*
********************************************************************************/
static void DiaperCheckInputDevices( void )
{
   if ( LButtonPressed )
   {
      LButtonPressed = FALSE;
      DiaperStart();                        // Launch a new diaper!
   }

   // Ignore all other mouse button events
   if ( LButtonReleased )
      LButtonReleased = FALSE;
   if ( RButtonPressed )
      RButtonPressed = FALSE;
   if ( RButtonReleased )
      RButtonReleased = FALSE;

   // Check for cheat codes
   DiaperCheckCheatCodes();
}


/******************************************************************************
*
*  NAME
*     DiaperCheckCheatCodes
*
*  SYNOPSIS
*     DiaperCheckCheatCodes()
*
*  DESCRIPTION
*     Checks for cheat codes typed in from the keyboard.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     DiaperCheckInputDevices()
*
********************************************************************************/
static void DiaperCheckCheatCodes( void )
{   
   if ( ( szCode[ 0 ] == 'd' ) && !strcmp( szCode, "defense" ) )
   {
      nEffectsDefended ^= TRUE;
      if ( !nEffectsDefended )
         g_defense.fxObject.dwFlags = ERASE;
      ResetCode();
   }
}


/******************************************************************************
*
*  NAME
*     DiaperStart
*
*  SYNOPSIS
*     DiaperStart()
*
*  DESCRIPTION
*     Launches a new diaper aimed at the current cursor position.
*     Calculates the diaper's initial velocity in order to hit the target.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*		Assumes there are 5 or 6 frames of animation for the diaper spinning sequences.
*     Assumes there are 2 different no ammo sounds.
*
*  SEE ALSO
*     DiaperCheckInputDevices()
*
********************************************************************************/
static void DiaperStart( void )
{      
   diaper_t *pDiaper, *pDiaperEnd;
   DWORD    dwDiaperCount;
   WORD     wFrameDelay;


   // Make sure launcher is done launching the previous diaper
   if ( s_fxLauncher.nFlags )
	   return;

   // Find room in object array to toss another diaper
   dwDiaperCount  = 0;
   pDiaper        = &s_aDiaper[ 0 ];
   pDiaperEnd     = &s_aDiaper[ MAX_OBJECTS_DIAPER ];
   while ( pDiaper < pDiaperEnd )
   {
      if ( pDiaper->dwState )
      {
         dwDiaperCount++;
         if ( dwDiaperCount == MAX_OBJECTS_DIAPER )
            return;              // Can't toss any more diapers for now
      }
      else
      {
         break;                  // Found a free spot, launch a new diaper
      }

      pDiaper++;
   }

   // Start the diaper launch sequence
   memset( pDiaper, 0, sizeof( *pDiaper ) );

   pDiaper->pfxDiaper = CreateFXObject();
   if ( !( pDiaper->pfxDiaper ) )
   {
      // Too many objects on screen already, play no ammo sound
      PlayWave( s_aiSounds[ SOUND_NO_AMMO_0 + ( rand() & 1 ) ] ); // !!! Assumes there are 2 different no ammo sounds
      return;
   }

   pDiaper->dwState        = DIAPER_STATE_LAUNCHING;
   s_fxLauncher.nFlags     = 1;                       // Start opening the launcher's lid
   s_fxLauncher.nFrameTime = 0;
   s_fxLauncher.dwFlags    = ( DWORD ) pDiaper;       // Let launcher know to launch this diaper object
   s_fxLauncher.dwFlags2   = ( nMouseY * 3 ) / nScreenHeight;  // Index into s_awLauncherRecoilDistances[]
   s_fxLauncher.y          += s_awLauncherRecoilDistances[ s_fxLauncher.dwFlags2 ];      // Recoil backwards

   // The higher the target, the higher the frame delay and hence the longer
   // it takes for the diaper to reach its target.
   wFrameDelay = s_awFrameDelays[ ( nMouseY * NUM_OF_FRAME_DELAYS ) / nScreenHeight ];
   pDiaper->wFrameDelay = wFrameDelay;

   InitObject( FindEffect( EFFECT_NAME_DIAPER ), pDiaper->pfxDiaper,
               &s_aTextures[ TEXTURE_DIAPER_SPIN_A_0 ],
               nMouseX - ( s_aTextures[ TEXTURE_DIAPER_SPIN_A_0 ].width >> 1 ),
               DIAPER_STARTING_Y );
   pDiaper->pfxDiaper->fScaleY   = ( float ) DIAPER_STARTING_Y;       // Keep track of y-position as a float
   pDiaper->pfxDiaper->nFrame    = ( rand() & 1 ) ? TEXTURE_DIAPER_SPIN_A_0 : TEXTURE_DIAPER_SPIN_B_0;
   pDiaper->pfxDiaper->dwFlags   = ( DWORD ) nMouseX; // Target's center x position
   pDiaper->pfxDiaper->dwFlags2  = ( DWORD ) nMouseY; // Target's center y position

   // Calculate the initial y-velocity of the diaper, positive y points upward
   // d = vt + a( t^2 )/2
   // v = ( d - a( t^2)/2 ) / t
   // d = ( DIAPER_STARTING_Y - ( nMouseY - s_aTextures[ TEXTURE_DIAPER_SPIN_A_0 ].height / 2 ) )
   // t = time to display all 5 frames of animation = 5 * wFrameDelay
   if ( pDiaper->pfxDiaper->nFrame == TEXTURE_DIAPER_SPIN_A_0 )
      pDiaper->pfxDiaper->fAngle = ( float )
         ( ( DIAPER_STARTING_Y - ( nMouseY - DIAPER_HOT_SPOT_Y ) )
           + ( G_ACCELERATION * ( 12.5f * wFrameDelay * wFrameDelay ) ) ) / ( 5.0f * wFrameDelay );
   else
      pDiaper->pfxDiaper->fAngle = ( float )
         ( ( DIAPER_STARTING_Y - ( nMouseY - DIAPER_HOT_SPOT_Y ) )
           + ( G_ACCELERATION * ( 18.0f * wFrameDelay * wFrameDelay ) ) ) / ( 6.0f * wFrameDelay );

   PlayWave( s_aiSounds[ SOUND_LAUNCH ] );
}


/******************************************************************************
*
*  NAME
*     DiaperCalculate
*
*  SYNOPSIS
*     DiaperCalculate()
*
*  DESCRIPTION
*     Calculates the position and appropriate frame to display for all
*     objects on the screen.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     Assumes there are 2 different final textures.
*
*  SEE ALSO
*     DiaperDraw(), DiaperDefenseMove(), DiaperDefenseCheckForCollision()
*
********************************************************************************/
static void DiaperCalculate( void )
{
   diaper_t    *pDiaper, *pDiaperEnd;
   FXOBJECT    *pfx;


   // ----- Update Diaper objects
   pDiaper     = &s_aDiaper[ 0 ];
   pDiaperEnd  = &s_aDiaper[ MAX_OBJECTS_DIAPER ];
   while ( pDiaper < pDiaperEnd )
   {
      pfx = pDiaper->pfxDiaper;
      switch( pDiaper->dwState )
      {
         case DIAPER_STATE_INVALID:
         case DIAPER_STATE_LAUNCHING:
            break;

         case DIAPER_STATE_SPINNING:
            pfx->nFrameTime++;

            // Update velocity and position of diaper as it travels through the air
            pfx->fAngle    -= ( float ) G_ACCELERATION;   // Positive velocity points upward
            pfx->fScaleY   -= pfx->fAngle;
            pfx->y         = ( int )( pfx->fScaleY );

            if ( !( pfx->nFrameTime % pDiaper->wFrameDelay ) )
            {
               pfx->nFrame++;
               if ( ( pfx->nFrame == TEXTURE_DIAPER_SPIN_B_0 ) ||
                    ( pfx->nFrame == TEXTURE_SPLAT_A_0 ) )
               {
                  pfx->nFrame--;    // Restore to last frame of spinning animation
                  pDiaper->dwState = DIAPER_STATE_SPLATTERING;
                  PlayWave( s_aiSounds[ SOUND_HIT_PAGE ] );
               }
            }
            break;

         case DIAPER_STATE_SPLATTERING:
            // Hmm...transitioning between different frame delays is not
            // guaranteed to be smooth...
            if ( ( pfx->nFrame == TEXTURE_DIAPER_SPIN_A_4 ) ||
                 ( pfx->nFrame == TEXTURE_DIAPER_SPIN_B_5 ) )
            {
	            // Recenter image over original target
	            pfx->x = pfx->dwFlags - ( s_aTextures[ TEXTURE_SPLAT_A_0 ].width >> 1 );
	            pfx->y = pfx->dwFlags2 - ( s_aTextures[ TEXTURE_SPLAT_A_0 ].height >> 1 );
	            pfx->nFrame = ( rand() & 1 ) ? TEXTURE_SPLAT_A_0 : TEXTURE_SPLAT_B_0;    // Assumes 2 different final textures
	         }

            pfx->nFrameTime++;
            if ( !( pfx->nFrameTime % DEFAULT_ANIM_FRAME_DELAY ) )
            {
               pfx->nFrame++;
               if ( ( pfx->nFrame == TEXTURE_SPLAT_B_0 ) || 
                    ( pfx->nFrame == TEXTURE_DEFENSE ) )
               {
                  pfx->nFrame--;
                  pDiaper->dwState = DIAPER_STATE_HIT_PAGE;
               }
            }
            break;

         case DIAPER_STATE_HIT_PAGE:
            break;

         case DIAPER_STATE_BLOCKED:
            // Update velocity and position of diaper as it travels through the air
            pfx->fAngle    -= ( float ) G_ACCELERATION;   // Positive velocity points upward
            pfx->fScaleY   -= pfx->fAngle;
            pfx->y         = ( int )( pfx->fScaleY );
            break;

         default:
            break;
      }

      pDiaper++;
   }

   // Update diaper launcher's animation frame and launch the diaper
   // when the lid is fully open
   if ( s_fxLauncher.nFlags > 0 )
   {
      s_fxLauncher.nFrameTime++;
      if ( !( s_fxLauncher.nFrameTime % DEFAULT_ANIM_FRAME_DELAY ) )
      {
         // Launcher is opening its lid
         s_fxLauncher.nFrame++;
         if ( ( s_fxLauncher.nFrame == TEXTURE_LAUNCHER_MED_CLOSED ) ||
              ( s_fxLauncher.nFrame == TEXTURE_LAUNCHER_HIGH_CLOSED ) ||
           	  ( s_fxLauncher.nFrame == TEXTURE_LAUNCHER_SMOKE_0 ) )
         {
            s_fxLauncher.nFrame--;
            s_fxLauncher.nFlags = -1;              // Start playing animation backwards

            // Start the diaper spinning sequence
            if ( s_fxLauncher.dwFlags )
               ( ( diaper_t * )( s_fxLauncher.dwFlags ) )->dwState = DIAPER_STATE_SPINNING;
            s_fxLauncher.dwFlags = 0;
         
            // Start smoke sequence, adjust smoke y for different cannon positions
            if ( !s_fxSmoke.nFlags )
            {
               s_fxSmoke.nFlags  = 1;
               s_fxSmoke.x       = s_fxLauncher.x + ( ( s_aTextures[ TEXTURE_LAUNCHER_LOW_CLOSED ].width - s_aTextures[ TEXTURE_LAUNCHER_SMOKE_0 ].width ) >> 1 );
               s_fxSmoke.y       = SMOKE_Y;
               if ( s_fxLauncher.nFrame == TEXTURE_LAUNCHER_MED_OPEN_WIDE )
                  s_fxSmoke.y -= 36;
               else if ( s_fxLauncher.nFrame == TEXTURE_LAUNCHER_HIGH_OPEN_WIDE )
                  s_fxSmoke.y -= 58;
            }
         }
      }
   }
   else if ( s_fxLauncher.nFlags < 0 )
   {
      s_fxLauncher.nFrameTime++;
      if ( !( s_fxLauncher.nFrameTime % DEFAULT_ANIM_FRAME_DELAY ) )
      {
         // Launcher is closing its lid
         s_fxLauncher.nFrame--;
         s_fxLauncher.y -= ( s_awLauncherRecoilDistances[ s_fxLauncher.dwFlags2 ] / 2 );    // Recover from recoil slowly
         if ( ( s_fxLauncher.nFrame == TEXTURE_LAUNCHER_LOW_CLOSED ) ||
              ( s_fxLauncher.nFrame == TEXTURE_LAUNCHER_MED_CLOSED ) ||
              ( s_fxLauncher.nFrame == TEXTURE_LAUNCHER_HIGH_CLOSED ) )
	      {
	         s_fxLauncher.nFlags  = 0;   // Launcher animation sequence is done
	         s_fxLauncher.y       = LAUNCHER_Y;  // Shouldn't be necessary, but just in case...
         }
      }
   }

   // Update diaper launcher's position if it's not animating
   if ( !s_fxLauncher.nFlags )
   {
      // Launcher is idle, adjust its position and frame depending on the
      // current position of the mouse cursor.
      s_fxLauncher.x = nMouseX - ( s_aTextures[ TEXTURE_LAUNCHER_LOW_CLOSED ].width >> 1 );
      if ( nMouseY < ( nScreenHeight / 3 ) )
         s_fxLauncher.nFrame = TEXTURE_LAUNCHER_HIGH_CLOSED;
      else if ( nMouseY < ( ( nScreenHeight * 2 ) / 3 ) )
         s_fxLauncher.nFrame = TEXTURE_LAUNCHER_MED_CLOSED;
      else
         s_fxLauncher.nFrame = TEXTURE_LAUNCHER_LOW_CLOSED;
   }

   // Update smoke animation
   if ( s_fxSmoke.nFlags > 0 )
   {
      s_fxSmoke.nFrameTime++;
      if ( !( s_fxSmoke.nFrameTime % ANIM_FRAME_DELAY_SMOKE ) )
      {
         s_fxSmoke.nFrame++;
         if ( s_fxSmoke.nFrame > TEXTURE_LAUNCHER_SMOKE_3 )
         {
            s_fxSmoke.nFlags     = ERASE;
            s_fxSmoke.nFrameTime = 0;
            s_fxSmoke.nFrame     = TEXTURE_LAUNCHER_SMOKE_0;
         }
      }
   }

   if ( g_defense.byObjectAlive )
   {
      DiaperDefenseCheckForCollision();
      DiaperDefenseMove();
   }
}


/******************************************************************************
*
*  NAME
*     DiaperDrawBuffer
*
*  SYNOPSIS
*     DiaperDrawBuffer( pDDrawData )
*
*  DESCRIPTION
*     Draws all objects into the offscreen buffer.
*
*  INPUTS
*     DDRAW_DATA *pDDrawData = Current DirectDraw device info.
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     DiaperDraw()
*
********************************************************************************/
static void DiaperDrawBuffer( DDRAW_DATA *pDDrawData )
{
   diaper_t    *pDiaper, *pDiaperEnd;
   FXOBJECT    *pfx;


   // Draw defense
   if ( g_defense.byObjectAlive )
      DrawObjectToBuffer( pDDrawData, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ], 0 );

   // Draw diapers
   pDiaper     = &s_aDiaper[ 0 ];
   pDiaperEnd  = &s_aDiaper[ MAX_OBJECTS_DIAPER ];
   while ( pDiaper < pDiaperEnd )
   {
      pfx = pDiaper->pfxDiaper;
      switch( pDiaper->dwState )
      {
         case DIAPER_STATE_INVALID:
            break;

         case DIAPER_STATE_SPINNING:
         case DIAPER_STATE_SPLATTERING:
            DrawObjectToBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ], 0 );
            break;

         case DIAPER_STATE_HIT_PAGE:
            DrawObjectToBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ], 0 );
            DrawObjectToScreenCapture( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ], 0 );
            break;

         case DIAPER_STATE_BLOCKED:
            DrawObjectToBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ], 0 );
            break;

         default:
            break;
      }

      pDiaper++;
   }

   // Draw smoke, if applicable
   if ( s_fxSmoke.nFlags == 1 )
   {
      DrawObjectToBuffer( pDDrawData, &s_fxSmoke, &s_aTextures[ s_fxSmoke.nFrame ], FAST_TRANSLUCENT_IMAGE );
   }
   else if ( s_fxSmoke.nFlags == ERASE )
   {
      // Draw diaper cannon
      DrawObjectToBuffer( pDDrawData, &s_fxLauncher, &s_aTextures[ s_fxLauncher.nFrame ], 0 );
      EraseObjectOnScreen( pDDrawData, &s_fxSmoke, &s_aTextures[ s_fxSmoke.nFrame ] );
      s_fxSmoke.nFlags = 0;         // Smoke has been erased, we're ready to draw new smoke
      return;
   }

   // Draw diaper cannon
   DrawObjectToBuffer( pDDrawData, &s_fxLauncher, &s_aTextures[ s_fxLauncher.nFrame ], 0 );
}


/******************************************************************************
*
*  NAME
*     DiaperDrawScreen
*
*  SYNOPSIS
*     DiaperDrawScreen( pDDrawData )
*
*  DESCRIPTION
*     Draws all objects from the offscreen buffer onto the front screen buffer.
*
*  INPUTS
*     DDRAW_DATA *pDDrawData = Current DirectDraw device info.
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     DiaperDraw()
*
********************************************************************************/
static void DiaperDrawScreen( DDRAW_DATA *pDDrawData )
{
   diaper_t *pDiaper, *pDiaperEnd;
   FXOBJECT *pfx;


   // Draw defense
   if ( g_defense.byObjectAlive )
      DrawBufferObjectToScreen( pDDrawData, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ] );

   // Draw diapers
   pDiaper     = &s_aDiaper[ 0 ];
   pDiaperEnd  = &s_aDiaper[ MAX_OBJECTS_DIAPER ];
   while ( pDiaper < pDiaperEnd )
   {
      pfx = pDiaper->pfxDiaper;
      switch( pDiaper->dwState )
      {
         case DIAPER_STATE_INVALID:
            break;

         case DIAPER_STATE_SPINNING:
         case DIAPER_STATE_SPLATTERING:
         case DIAPER_STATE_HIT_PAGE:
         case DIAPER_STATE_BLOCKED:
            DrawBufferObjectToScreen( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
            break;

         default:
            break;
      }

      pDiaper++;
   }

   // Draw smoke, if applicable
   if ( s_fxSmoke.nFlags == 1 )
      DrawBufferObjectToScreen( pDDrawData, &s_fxSmoke, &s_aTextures[ s_fxSmoke.nFrame ] );

   // Draw diaper cannon
   DrawBufferObjectToScreen( pDDrawData, &s_fxLauncher, &s_aTextures[ s_fxLauncher.nFrame ] );

   // Erase the defense
   if ( g_defense.fxObject.dwFlags == ERASE )
   {
      EraseObjectInBuffer( pDDrawData, &( g_defense.fxObject ), &s_aTextures[ TEXTURE_DEFENSE ] );
      EraseObjectOnScreen( pDDrawData, &( g_defense.fxObject ), &s_aTextures[ TEXTURE_DEFENSE ] );
      g_defense.fxObject.dwFlags = 0;
      g_defense.fxObject.nFlags  = DEFENSE_STATE_SEEKING;   // Reset the defense mechanism
   }
}


/******************************************************************************
*
*  NAME
*     DiaperEraseBuffer
*
*  SYNOPSIS
*     DiaperEraseBuffer( pDDrawData )
*
*  DESCRIPTION
*     Erases all the objects that were previously drawn into the offscreen
*     buffer in order to restore the offscreen buffer back to its original state
*     so we can draw on a clean background again on the next iteration of
*     the drawing pipeline.
*
*     This also handles the ending state transitions, except for the defense mechanism.
*
*  INPUTS
*     DDRAW_DATA *pDDrawData = Current DirectDraw device info.
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     DiaperDraw()
*
********************************************************************************/
static void DiaperEraseBuffer( DDRAW_DATA *pDDrawData )
{
   diaper_t *pDiaper, *pDiaperEnd;
   FXOBJECT *pfx;


   // Erase diapers
   pDiaper     = &s_aDiaper[ 0 ];
   pDiaperEnd  = &s_aDiaper[ MAX_OBJECTS_DIAPER ];
   while ( pDiaper < pDiaperEnd )
   {
      pfx = pDiaper->pfxDiaper;
      switch( pDiaper->dwState )
      {
         case DIAPER_STATE_INVALID:
            break;

         case DIAPER_STATE_SPINNING:
         case DIAPER_STATE_SPLATTERING:
            EraseObjectInBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
            break;

         case DIAPER_STATE_HIT_PAGE:
            EraseObjectInBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
            pDiaper->dwState     = DIAPER_STATE_INVALID;
            pfx->nFlags          = 0;     // Clear drawing flags for automatically saved object
            pDiaper->pfxDiaper   = NULL;  // Note that the global Projectile[] array will still contain the
                                          // exploded hole from this diaper, which is what we want
            break;

         case DIAPER_STATE_BLOCKED:
            EraseObjectInBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
            if ( pfx->y > nScreenHeight )
            {
//               EraseObjectOnScreen( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
               pfx->x = -9999;   // Kluge, we move the object out of bounds so it won't be drawn.
               pfx->y = -9999;   // The entry in the global Projectiles[] array will be erased later
                                 // when the effect is ended and the array is condensed
               pDiaper->dwState     = DIAPER_STATE_INVALID;
               pDiaper->pfxDiaper   = NULL;
            }
            break;

         default:
            break;
      }

      pDiaper++;
   }

   // Erase smoke
   EraseObjectInBuffer( pDDrawData, &s_fxSmoke, &s_aTextures[ s_fxSmoke.nFrame ] );

   // Erase diaper cannon
   EraseObjectInBuffer( pDDrawData, &s_fxLauncher, &s_aTextures[ s_fxLauncher.nFrame ] );

   // Erase defense
   if ( g_defense.byObjectAlive )
      EraseObjectInBuffer( pDDrawData, &( g_defense.fxObject ), &s_aTextures[ TEXTURE_DEFENSE ] );
}


/******************************************************************************
*
*  NAME
*		DiaperDefenseMove
*
*  SYNOPSIS
*     DiaperDefenseMove()
*
*  DESCRIPTION
*     Determines where the umbrella should move for the time being.
*     The umbrella will seek the first diaper scheduled to hit the page,
*     or it simply follows the cursor if there are no diapers being thrown.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     None
*
*  SEE ALSO
*     DiaperCalculate(), DiaperDefenseCheckForCollision()
*
********************************************************************************/
static void DiaperDefenseMove( void )
{
   diaper_t *pDiaper, *pDiaperEnd;
   FXOBJECT *pfx;


   if ( g_defense.fxObject.nFlags == DEFENSE_STATE_SEEKING )
   {
      int         dx, dy;
      int         nFrameTarget;


      // Find a new target, which can be either the mouse cursor, 
      // or the first diaper scheduled to hit the page.
      // The first diaper to hit the page is simply the one with the
      // highest nFrameTime member, which indicates how long it's been alive.
      g_defense.nDestX  = DEFENSE_FOLLOW_CURSOR;   // Assume no diapers on screen for now
      nFrameTarget      = -1;
      pDiaper           = &s_aDiaper[ 0 ];
      pDiaperEnd        = &s_aDiaper[ MAX_OBJECTS_DIAPER ];
      while ( pDiaper < pDiaperEnd )
      {
         pfx = pDiaper->pfxDiaper;
         if ( ( pDiaper->dwState == DIAPER_STATE_SPINNING ) ||
              ( pDiaper->dwState == DIAPER_STATE_SPLATTERING ) )
         {
            if ( pfx->nFrameTime > nFrameTarget )
            {
               nFrameTarget      = pfx->nFrameTime;
               g_defense.nDestX  = ( int )( pfx->dwFlags );
               g_defense.nDestY  = ( int )( pfx->dwFlags2 ) + DIAPER_HOT_SPOT_Y - s_aTextures[ TEXTURE_SPLAT_A_0 ].height; // Aim for where diaper hits the page, not where it rolls down to
            }
         }

         pDiaper++;
      }

      if ( g_defense.nDestX == DEFENSE_FOLLOW_CURSOR )
      {
         g_defense.nDestX = nMouseX;
         g_defense.nDestY = nMouseY + DIAPER_HOT_SPOT_Y - s_aTextures[ TEXTURE_SPLAT_A_0 ].height;
      }

      g_defense.nSpeed = ( rand() % ( DEFENSE_MAX_STEP - DEFENSE_MIN_STEP + 1 ) ) + DEFENSE_MIN_STEP;

      // Umbrella does not span the entire surface of the bitmap
      // So we use hardcoded values for the actual width and height
      dx = g_defense.nDestX - ( g_defense.fxObject.x + DEFENSE_X1 + ( DEFENSE_X2 >> 1 ) );
      dy = g_defense.nDestY - ( g_defense.fxObject.y + DEFENSE_Y1 + ( DEFENSE_Y2 >> 1 ) );

      g_defense.fIncrX = ( float ) dx / ( float ) g_defense.nSpeed;
      g_defense.fIncrY = ( float ) dy / ( float ) g_defense.nSpeed;

      g_defense.nCount = 0;      // This keeps track of which step we're on
      g_defense.fxObject.nFlags = DEFENSE_STATE_MOVING;
   }

   // ----- Move the defense if applicable
   if ( g_defense.fxObject.nFlags == DEFENSE_STATE_MOVING )
   {
      g_defense.fxObject.x += ( int ) g_defense.fIncrX;
      g_defense.fxObject.y += ( int ) g_defense.fIncrY;

      g_defense.nCount++;                          // Increment the current step counter
      if ( g_defense.nCount > g_defense.nSpeed )   // We've reached our target
         g_defense.fxObject.nFlags = DEFENSE_STATE_SEEKING;
   }
}


/******************************************************************************
*
*  NAME
*		DiaperDefenseCheckForCollision
*
*  SYNOPSIS
*     DiaperDefenseCheckForCollision()
*
*  DESCRIPTION
*     Checks to see if the umbrella has blocked a diaper.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     Assumes there are 3 different damage textures.
*
*  SEE ALSO
*     DiaperCalculate(), DiaperDefenseMove()
*
********************************************************************************/
static void DiaperDefenseCheckForCollision( void )
{
   diaper_t *pDiaper, *pDiaperEnd;
   TEXTURE  *pTextureUmbrella, *pTextureDamage;


   pTextureUmbrella  = &s_aTextures[ TEXTURE_DEFENSE ];
   pTextureDamage    = &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_0 + ( rand() % 3 ) ];   // Pick one of three different damage textures

   // ----- Determine if any objects have just hit the umbrella
   pDiaper     = &s_aDiaper[ 0 ];
   pDiaperEnd  = &s_aDiaper[ MAX_OBJECTS_DIAPER ];
   while ( pDiaper < pDiaperEnd )
   {
      if ( ( pDiaper->dwState == DIAPER_STATE_HIT_PAGE ) ||
           ( pDiaper->dwState == DIAPER_STATE_SPLATTERING ) )
      {
         int         iDefenseX1, iDefenseX2, iDefenseY1, iDefenseY2;
         int         iWeaponX1, iWeaponX2, iWeaponY1, iWeaponY2;
         FXOBJECT *  pfxObject;


         pfxObject = pDiaper->pfxDiaper;

         // Check for collision using bounding boxes.
         iDefenseX1 = g_defense.fxObject.x + DEFENSE_X1;
         iDefenseX2 = g_defense.fxObject.x + DEFENSE_X2;
         iDefenseY1 = g_defense.fxObject.y + DEFENSE_Y1;
         iDefenseY2 = g_defense.fxObject.y + DEFENSE_Y2;

         iWeaponX1 = pfxObject->x + DIAPER_COLLISION_BOX_X1;
         iWeaponX2 = pfxObject->x + DIAPER_COLLISION_BOX_X2;
         iWeaponY1 = pfxObject->y + DIAPER_COLLISION_BOX_Y1;
         iWeaponY2 = pfxObject->y + DIAPER_COLLISION_BOX_Y2;

         if ( ( iDefenseX1 >= iWeaponX2 ) || ( iDefenseX2 <= iWeaponX1 ) ||
              ( iDefenseY1 >= iWeaponY2 ) || ( iDefenseY2 <= iWeaponY1 ) )
            goto next;        // Trivial reject, rectangles do not intersect at all

    	   // Weapon collides with the defense!
         // Center damage image at target x and y relative to umbrella
#if 0
         g_defense.ptDef.x = ( int )( pfxObject->dwFlags ) - ( pTextureDamage->width >> 1 ) - g_defense.fxObject.x;
         g_defense.ptDef.y = ( int )( pfxObject->dwFlags2 ) - ( pTextureDamage->height >> 1 ) - g_defense.fxObject.y;
#else
         g_defense.ptDef.x = ( int )( pfxObject->x + ( s_aTextures[ TEXTURE_DIAPER_SPIN_A_4 ].width >> 1 ) ) - ( pTextureDamage->width >> 1 ) - g_defense.fxObject.x;
         g_defense.ptDef.y = ( int )( pfxObject->y + ( s_aTextures[ TEXTURE_DIAPER_SPIN_A_4 ].height >> 1 ) ) - ( pTextureDamage->height >> 1 ) - g_defense.fxObject.y;
#endif

         if ( ( g_defense.ptDef.x >= DEFENSE_DRAW_X1 ) &&
              ( ( g_defense.ptDef.x + pTextureDamage->width ) <= DEFENSE_DRAW_X2 ) &&
              ( g_defense.ptDef.y >= DEFENSE_DRAW_Y1 ) &&
              ( ( g_defense.ptDef.y + pTextureDamage->height ) <= DEFENSE_DRAW_Y2 ) )
         {
            DrawTextureInTexture( pTextureUmbrella, pTextureDamage, &g_defense.ptDef );
         }

         pDiaper->dwState           = DIAPER_STATE_BLOCKED;
         pDiaper->pfxDiaper->fAngle = 12.0f;                   // Give diaper some upward velocity at the point of impact
         g_defense.fxObject.nFlags  = DEFENSE_STATE_SEEKING;   // Seek new target
         PlayWave( s_aiSounds[ SOUND_BLOCKED ] );
      }
next:
      pDiaper++;
   }

}


/******************************************************************************
*
*  NAME
*		DiaperManualSave
*
*  SYNOPSIS
*     iSuccess = DiaperManualSave( pDDrawData, iMsg, pvData )
*
*  DESCRIPTION
*     Takes care of displaying the correct final image for the diaper effect.
*     Also loads and saves effect data in the same manner as the manual save
*     functions for the other weapon packs.
*
*  INPUTS
*     DDRAW_DATA *   pDDrawData  = Pointer to DirectDraw device.
*     int            iMsg        = Message to process, i.e. load, save, or display.
*     void *         pvData      = Data location for reads or writes.
*
*  OUTPUTS
*     int iSuccess = TRUE
*
*  BUGS
*     Assumes 2 different final textures, which is hardcoded instead of using a macro.
*
*  SEE ALSO
*
********************************************************************************/
int DiaperManualSave( DDRAW_DATA *pDDrawData, int iMsg, void *pvData )
{
   static   BOOL              s_bTexturesLoaded = FALSE;
   static   TEXTURE           s_aSaveTextures[ 2 ] = { { 0 }, { 0 } };
            TEXTURE *         pTexture;
            FXOBJECT *        pfxObject;
            URLDATA *         pURLData;
            diaperSaveData_t  *pSaveData, *pSaveDataEnd;
            int               iEffectID;


   iEffectID = FindEffect( EFFECT_NAME_DIAPER );

   switch( iMsg )
	{
	case MANUAL_DRAW_TO_BROWSER:
      if ( !s_bTexturesLoaded )
      {
         // Make sure these texture file names match with the LoadTexture() calls above
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatb06.pcx", &s_aSaveTextures[ 0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Diaper\\7esplatc06.pcx", &s_aSaveTextures[ 1 ] );
         s_bTexturesLoaded = TRUE;
      }

		pfxObject   = ( FXOBJECT * ) pvData;
      if ( ( pfxObject->nFrame != TEXTURE_SPLAT_A_5 ) && 
           ( pfxObject->nFrame != TEXTURE_SPLAT_B_5 ) )
         break;

      pTexture = &s_aSaveTextures[ ( pfxObject->nFrame == TEXTURE_SPLAT_A_5 ) ? 0 : 1 ];
      DrawObjectToBrowserBuffer( pDDrawData, pfxObject, pTexture, 0 );
		break;

   case MANUAL_LOADED_EFFECT_DATA:
      pURLData       = ( URLDATA * ) pvData;
      pSaveData      = ( diaperSaveData_t * )( pURLData->ptrData );
		pSaveDataEnd   = ( diaperSaveData_t * )( ( DWORD ) pSaveData + ( DWORD ) ( pURLData->nSize ) );

      while ( pSaveData < pSaveDataEnd )
      {
         pfxObject = CreateFXObject();
         if ( !pfxObject )
            break;

         InitObject( iEffectID, pfxObject, NULL, 0, 0 );
         pfxObject->x               = pSaveData->x;
         pfxObject->y               = pSaveData->y;
         pfxObject->nFrame          = pSaveData->iFrame;
         pfxObject->nInsideBrowser  = TRUE;

         pSaveData++;
      }

      free( pURLData->ptrData ); // This is not very kosher!
      break;

	case MANUAL_SAVE_EFFECT_DATA:
      pURLData          = ( URLDATA * ) pvData;
      pURLData->nSize   = GetTotalFXObjectsForEffect( iEffectID ) * sizeof( diaperSaveData_t );
      pURLData->ptrData = malloc( pURLData->nSize );
      pSaveData         = ( diaperSaveData_t * )( pURLData->ptrData );
      pSaveDataEnd      = ( diaperSaveData_t * )( ( DWORD ) pSaveData + ( DWORD )( pURLData->nSize ) );
      if ( !pSaveData )
         break;                                 // Error! malloc() failed!

      pfxObject = GetNextFXObject( iEffectID, FIND_FIRST_OBJECT );
      if ( !pfxObject )
         break;

      pSaveData->x      = pfxObject->x;
      pSaveData->y      = pfxObject->y;
      pSaveData->iFrame = pfxObject->nFrame;
      pSaveData++;

      while ( pSaveData < pSaveDataEnd )
      {
         pfxObject = GetNextFXObject( iEffectID, FIND_NEXT_OBJECT );
         if ( !pfxObject )
            break;

     		pSaveData->x      = pfxObject->x;
     		pSaveData->y      = pfxObject->y;
     		pSaveData->iFrame = pfxObject->nFrame;

         pSaveData++;
    	}
    	break;
	}

   return ( TRUE );
}



