/*******************************************************************************
*
*  Web Vengeance Poo Pack - seagull.c
*
*  The Poo Pack's Seagull effect routines.
*
*	Copyright (c) 1997 SegaSoft, Inc.  All Rights Reserved.
*
*------------------------------------------------------------------------------
*                   CONFIDENTIAL AND PROPRIETARY INFORMATION
*------------------------------------------------------------------------------
*  The contents of this file are the property of SegaSoft, Inc., 
*  and may not be used or duplicated without the permission of SegaSoft, Inc.
*------------------------------------------------------------------------------
*
*  Edit History:
*		97-Sep-29	DJS	Creation.
*
*  Initials:
*  	DJS	Dan J. Su
*
*******************************************************************************/
#include <ddraw.h>
#include <windows.h>
#include <math.h>
#include <stdlib.h>
#include "BlasterEffects.h"
#include "EffectType.h"

#include "main.h"
#define DEFINE_SEAGULL
#include "seagull.h"


// ---------- Defines
#define MAX_OBJECTS_SEAGULL      ( 5UL )  // Max number of seagulls on screen at once
#define MAX_OBJECTS_POOP         ( 6UL )  // Max number of falling poop from each seagull

#define PI                       ( 3.1415926535f )

#define ROTATE_MIN               (  20 )  // Minimum number of tenths of a degree to turn for each step forward
#define ROTATE_MAX               (  50 )  // Maximum number of tenths of a degree to turn for each step forward
#define TURNING_ANGLE_MIN        (  20 )  // Minimum degrees seagull will try to turn
#define TURNING_ANGLE_MAX        ( 180 )  // Maximum degrees seagull will try to turn
#define MIN_FLAPS                (   1 )  // Minimum number of times the seagull flaps its wings
#define MAX_FLAPS                (   2 )  // Maximum number of times the seagull flaps its wings

#define SQUAWK_DELAY             ( 700 )  // Minimum delay inbetween seagull squawks, in milliseconds
#define SQUAWK_PROBABILITY       (   4 )  // 0 to 100 percent chance of squawking, checked every 1/30th of a second

   // Anim frame delays, where we're playing at 30 FPS
#define ANIM_DELAY_CURSOR        (  2 )   // Animated cursor
#define ANIM_DELAY_FLAPPING      (  2 )   // Delay between flapping animation frames for the seagull
#define ANIM_DELAY_SEAGULL_MOVE  (  1 )   // Delay between moving the seagull
#define ANIM_DELAY_POOP_MOVE     (  1 )   // Delay between moving falling poop

#define TIME_TO_DROP_POOP        ( 32 )   // Time when we should drop the poop.
                                          // Must be less than TIME_TO_HIT_TARGET, 
                                          // in multiples of ANIM_DELAY_SEAGULL_MOVE.
#define TIME_TO_HIT_TARGET       ( 48 )   // Time when the poop will hit the target
                                          // Must be greater than TIME_TO_DROP_POOP,
                                          // in multiples of ANIM_DELAY_SEAGULL_MOVE

// ----- Texture indexes
#define TEXTURE_CURSOR_0         (  0 )   // First frame of animated cursor
#define TEXTURE_CURSOR_1         (  1 )
#define TEXTURE_CURSOR_2         (  2 )
#define TEXTURE_CURSOR_3         (  3 )
#define TEXTURE_CURSOR_4         (  4 )
#define TEXTURE_CURSOR_5         (  5 )   // Last frame of animated cursor
#define TEXTURE_SEAGULL_0        (  6 )   // Wings spread widest, feathers at tip of wing spread out
#define TEXTURE_SEAGULL_1        (  7 )   // Wings spread out
#define TEXTURE_SEAGULL_2        (  8 )   // Wings flapping inward
#define TEXTURE_SEAGULL_3        (  9 )   // Wings flapping down a lot
#define TEXTURE_SEAGULL_4        ( 10 )   // Wings tucked under
#define TEXTURE_FALLING_POOP_A   ( 11 )   // Falling birdie poop, type A
#define TEXTURE_FALLING_POOP_B   ( 12 )   // Falling birdie poop, type B
#define TEXTURE_FALLING_POOP_C   ( 13 )   // Falling birdie poop, type C
#define TEXTURE_SPLAT_A_0        ( 14 )   // Birdie poop hitting page, type A, frame 0
#define TEXTURE_SPLAT_A_1        ( 15 )
#define TEXTURE_SPLAT_A_2        ( 16 )
#define TEXTURE_SPLAT_A_3        ( 17 )   // Final frame to be saved on the page
#define TEXTURE_SPLAT_B_0        ( 18 )   // Birdie poop hitting page, type B
#define TEXTURE_SPLAT_B_1        ( 19 )
#define TEXTURE_SPLAT_B_2        ( 20 )
#define TEXTURE_SPLAT_B_3        ( 21 )   // Final frame
#define TEXTURE_SPLAT_C_0        ( 22 )   // Birdie poop hitting page, type C
#define TEXTURE_SPLAT_C_1        ( 23 )
#define TEXTURE_SPLAT_C_2        ( 24 )
#define TEXTURE_SPLAT_C_3        ( 25 )   // Final frame
#define TEXTURE_DEFENSE          ( 26 )   // Umbrella
#define TEXTURE_DEFENSE_DAMAGE_0 ( 27 )   // Damage mark for umbrella
#define TEXTURE_DEFENSE_DAMAGE_1 ( 28 )
#define TEXTURE_DEFENSE_DAMAGE_2 ( 29 )
#define NUM_TEXTURES_TOTAL       ( 30 )

   // ----- Sound indexes
#define SOUND_NO_AMMO            ( 0 )
#define SOUND_SQUAWK             ( 1 )    // Seagull squawking sound
#define SOUND_HIT_PAGE           ( 2 )    // Poop hits the page
#define SOUND_BLOCKED            ( 3 )    // Poop hits the umbrella
#define NUM_SOUNDS_TOTAL         ( 4 )

   // ----- Defense info
#define DEFENSE_STATE_SEEKING    ( 1 )    // Defense mechanism needs to find a new target, cursor or seagull
#define DEFENSE_STATE_MOVING     ( 2 )    // Defense mechanism is moving towards target
#define DEFENSE_FOLLOW_CURSOR    ( -999 ) // Pick an invalid x-coordinate to indicate
                                          // there are no seagulls to target
#define DEFENSE_MIN_STEP         ( 2 )    // Minimum # steps to reach destination
#define DEFENSE_MAX_STEP         ( 5 )    // Maximum # of steps to reach destination

   // Collision box offsets, relative to top-left corner of the TEXTURE_SPLAT_A_0 bitmap
   // This area should be relatively small and centered where the poop would hit the page
#define SEAGULL_COLLISION_BOX_X1 (  0 )
#define SEAGULL_COLLISION_BOX_X2 ( 64 )
#define SEAGULL_COLLISION_BOX_Y1 (  0 )
#define SEAGULL_COLLISION_BOX_Y2 ( 64 )

   // ----- States for the seagull effect
      // Flying seagull
#define SEAGULL_STATE_INVALID    ( 0 )    // Marks an unused seagull object
#define SEAGULL_STATE_APPROACH   ( 1 )    // Seagull is approaching target
#define SEAGULL_STATE_LEAVING    ( 2 )    // Seagull has dropped its load and is now leaving the target

      // Seagull's poop
#define POOP_STATE_INVALID       ( 0 )    // No poop for this seagull for now
#define POOP_STATE_FALLING       ( 1 )    // Poop is falling towards target
#define POOP_STATE_HIT_PAGE      ( 3 )    // Poop has hit the page, play through animation frames
#define POOP_STATE_HIT_PAGE_END  ( 4 )    // Done hitting the page
#define POOP_STATE_BLOCKED       ( 5 )    // Poop was blocked by umbrella


// ---------- Structures and Types
   // Each poop object contains the poop sprites for one seagull and
   // the pointer to the actual FXOBJECT used for the final poop image on the page.
typedef struct poop_s
{
   DWORD    dwState;                      // Current state we're in.  See POOP_STATE_... defines above.
   FXOBJECT *pfxPoop;                     // Pointer to FXOBJECT in main app's global Projectile[] array.
                                          // This is the hole, crater, etc. that gets saved for the current web page.
                                          // dwFlags member contains x-velocity
                                          // dwFlags2 member contains y-velocity
                                          // fScaleX and fScaleY are used for scaling
                                          // nFlags contains the amount to scale down the poop as it falls
} poop_t;

   // Each seagull object contains the FXOBJECT for the seagull and 
   // a poop struct for the poop ejected by the seagull.
typedef struct seagull_s
{
   DWORD       dwState;    // Current state we're in.  See SEAGULL_STATE_... defines above.
   FXOBJECT    fxSeagull;  // Flying seagull which dumps the poop
                           // nFlags member is used to indicate anim playback
                           // direction for the ponged seagull flapping animation.
                           // dwFlags member represents x-velocity.
                           // dwFlags2 member represents y-velocity.
   int         iTargetX;   // Target's x-position, this helps defense movement
   int         iTargetY;   // Target's y-position
   WORD        wNumFlaps;  // Number of times the seagull flaps its wings at the beginning
   WORD        wUnused;
   DWORD       dwTimeOfLastSquawk;  // Each seagull squawks at random intervals.
   float       fAngleDiff; // This represents how fast the seagull turns away after dropping its load, in degrees
   float       fAngleStart;// Keep track of starting angle so we know when to stop turning
   float       fAngleMax;  // The seagull will turn no more than this many degrees from its original trajectory
   float       fMagnitude; // Magnitude of the seagull's velocity
   poop_t      poopObject; // Contains poop info for this seagull
} seagull_t;


// ---------- Local Functions
static void    SeagullCheckInputDevices         ( void );
static void    SeagullCheckCheatCodes           ( void );
static void    SeagullStart                     ( void );
static double  SeagullGetCursorAngle            ( void );
static void    SeagullDropPoop                  ( seagull_t *pSeagull );
static void    SeagullCalculate                 ( void );
static void    SeagullDrawBuffer                ( DDRAW_DATA *pDDrawData );
static void    SeagullEraseBuffer               ( DDRAW_DATA *pDDrawData );
static void    SeagullDrawScreen                ( DDRAW_DATA *pDDrawData );
static void    SeagullDefenseMove               ( void );
static void    SeagullDefenseCheckForCollision  ( void );


// ---------- Local Variables
static seagull_t  s_aSeagulls[ MAX_OBJECTS_SEAGULL ]; // Array of seagull objects
static TEXTURE    s_aTextures[ NUM_TEXTURES_TOTAL ];  // Textures
static int        s_aiSounds[ NUM_SOUNDS_TOTAL ];     // Sound indexes


/******************************************************************************
*
*  NAME
*     SeagullUpdate
*
*  SYNOPSIS
*     iSuccess = SeagullUpdate( pDDrawData, iEffect, iMsg, iObjectCount );
*
*  DESCRIPTION
*     Starts or ends the seagull effect.
*
*  INPUTS
*     DDRAW_DATA *   pDDrawData     = Info on current DirectDraw device
*     int            iEffect        = Effect ID, should be the ID for the seagull effect
*     int            iMsg           = Message to process i.e. start or end effect
*     int            iObjectCount   = Not used
*
*  OUTPUTS
*		int iSuccess = TRUE if update was successful, FALSE otherwise
*
*  BUGS
*		None
*
*  SEE ALSO
*     InitEffects()
*
********************************************************************************/
BOOL SeagullUpdate( DDRAW_DATA *pDDrawData, int iEffect, int iMsg, int iObjectCount )
{
   if ( FindEffect( EFFECT_NAME_SEAGULL ) != iEffect )
      return ( FALSE );

   switch( iMsg )
   {
      case EFFECT_STARTED:
      {
         srand( GetTickCount() );

         // Load sounds
//			s_aiSounds[ SOUND_NO_AMMO ]   = LoadWave( ".\\Sounds\\Poo\\Seagull\\NoAmmo.wav" );
         s_aiSounds[ SOUND_SQUAWK ]    = LoadWave( ".\\Sounds\\Poo\\Seagull\\Squawk.wav" );
			s_aiSounds[ SOUND_HIT_PAGE ]  = LoadWave( ".\\Sounds\\Poo\\Seagull\\Splat.wav" );
			s_aiSounds[ SOUND_BLOCKED ]   = LoadWave( ".\\Sounds\\Poo\\Seagull\\Splat.wav" );   // !!! No special blocking sound for now

         // Load textures
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7cgull01.pcx",  &s_aTextures[ TEXTURE_CURSOR_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7cgull02.pcx",  &s_aTextures[ TEXTURE_CURSOR_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7cgull03.pcx",  &s_aTextures[ TEXTURE_CURSOR_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7cgull04.pcx",  &s_aTextures[ TEXTURE_CURSOR_3 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7cgull05.pcx",  &s_aTextures[ TEXTURE_CURSOR_4 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7cgull06.pcx",  &s_aTextures[ TEXTURE_CURSOR_5 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\Gull0.pcx",  &s_aTextures[ TEXTURE_SEAGULL_0 ] );      // wings spread out extra wide
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\Gull1.pcx",  &s_aTextures[ TEXTURE_SEAGULL_1 ] );      // wings spread out normal
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\Gull2.pcx",  &s_aTextures[ TEXTURE_SEAGULL_2 ] );      // wings flapping down a little
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\Gull3.pcx",  &s_aTextures[ TEXTURE_SEAGULL_3 ] );      // wings flapping down a lot
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\Gull4.pcx",  &s_aTextures[ TEXTURE_SEAGULL_4 ] );      // wings flapping all the way down

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7edropa.pcx",   &s_aTextures[ TEXTURE_FALLING_POOP_A ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7edropb.pcx",   &s_aTextures[ TEXTURE_FALLING_POOP_B ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7edropc.pcx",   &s_aTextures[ TEXTURE_FALLING_POOP_C ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanid01.pcx",   &s_aTextures[ TEXTURE_SPLAT_A_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanid02.pcx",   &s_aTextures[ TEXTURE_SPLAT_A_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanid03.pcx",   &s_aTextures[ TEXTURE_SPLAT_A_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanid04.pcx",   &s_aTextures[ TEXTURE_SPLAT_A_3 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanie01.pcx",   &s_aTextures[ TEXTURE_SPLAT_B_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanie02.pcx",   &s_aTextures[ TEXTURE_SPLAT_B_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanie03.pcx",   &s_aTextures[ TEXTURE_SPLAT_B_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanie04.pcx",   &s_aTextures[ TEXTURE_SPLAT_B_3 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanif01.pcx",   &s_aTextures[ TEXTURE_SPLAT_C_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanif02.pcx",   &s_aTextures[ TEXTURE_SPLAT_C_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanif03.pcx",   &s_aTextures[ TEXTURE_SPLAT_C_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanif04.pcx",   &s_aTextures[ TEXTURE_SPLAT_C_3 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Umbrella\\7dumb01.pcx",  &s_aTextures[ TEXTURE_DEFENSE ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanid01.pcx",  &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanie01.pcx",  &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanif01.pcx",  &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_2 ] );

         InitObject( FX_ID_CURSOR, &g_fxCursor, &s_aTextures[ TEXTURE_CURSOR_0 ],
      	            nMouseX - ( s_aTextures[ TEXTURE_CURSOR_0 ].width >> 1 ),
      	            nMouseY - ( s_aTextures[ TEXTURE_CURSOR_0 ].height >> 1 ) );
			g_fxCursor.nFrameTime   = 0;
         g_fxCursor.nFrame       = TEXTURE_CURSOR_0;

         // Defense object is an umbrella
         InitObject( FX_ID_DEFENSE, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ],
                     ( nScreenWidth >> 1 ), ( nScreenHeight >> 1 ) );
         g_defense.fxObject.nFlags  = DEFENSE_STATE_SEEKING;   // Reset state flag
         g_defense.fxObject.dwFlags = 0;                       // Clear erase flag

         // Init table of Seagull objects
         memset( s_aSeagulls, 0, sizeof( seagull_t ) * MAX_OBJECTS_SEAGULL );

         break;
      }

      case EFFECT_ENDED:
      {
         seagull_t   *pSeagull, *pSeagullEnd;
         int         i;


         DeleteWaves();       // Free sounds

         for ( i = 0; i < NUM_TEXTURES_TOTAL; i++ )
            DeleteTexture( &s_aTextures[ i ] );

         // Delete all objects that are in the middle of an animation sequence.
         // They don't count as hits on the page.
         pSeagull       = &s_aSeagulls[ 0 ];
         pSeagullEnd    = &s_aSeagulls[ MAX_OBJECTS_SEAGULL ];
         while ( pSeagull < pSeagullEnd )
         {
            if ( pSeagull->dwState && pSeagull->poopObject.pfxPoop )
            {
               pSeagull->dwState             = SEAGULL_STATE_INVALID;
               pSeagull->poopObject.pfxPoop  = NULL;
               DeleteLastFXObject();      // Remove record of this object from the global
                                          // Projectile[] array so it doesn't get saved.
                                          // This takes care of exiting in the middle of an animation sequence
            }

            pSeagull++;
         }
         break;
      }

   	case DRAW_CREDITS:   // Only one of the effects in this pack should draw credits
   	   DrawCredit( "Poo Pack", CREDIT_PACK_NAME );
   	   DrawCredit( "", 0 );
   	   DrawCredit( "Producer", CREDIT_TITLE );
   	   DrawCredit( "Michael Person", CREDIT_NAME );
   		DrawCredit( "", 0);
   	   DrawCredit( "Programmer", CREDIT_TITLE);
   	   DrawCredit( "Dan Su", CREDIT_NAME );
   		DrawCredit( "", 0 );
   		DrawCredit( "Lead Artist", CREDIT_TITLE );
   		DrawCredit( "John Broenen", CREDIT_NAME );
   		DrawCredit( "", 0 );
   		DrawCredit( "Artist", CREDIT_TITLE );
   		DrawCredit( "John Pedigo", CREDIT_NAME );
   		DrawCredit( "", 0 );
   		DrawCredit( "Sound Effects", CREDIT_TITLE );
   		DrawCredit( "Tristan T. des Pres", CREDIT_NAME );
         DrawCredit( "", 0 );
         DrawCredit( "Marketing", CREDIT_TITLE );
         DrawCredit( "Donalyn Mason", CREDIT_NAME );
         DrawCredit( "", 0 );
   		break;

      default:
         break;      // Future commands do not generate an error
   }

   return ( TRUE );
}


/******************************************************************************
*
*  NAME
*     SeagullDraw
*
*  SYNOPSIS
*     iSuccess = SeagullDraw( pDDrawData, iMsg );
*
*  DESCRIPTION
*     Draws the current frame for the seagull effect
*
*  INPUTS
*     DDRAW_DATA *   pDDrawData     = Info on current DirectDraw device
*     int            iMsg           = Message to process
*
*  OUTPUTS
*		int iSuccess = TRUE
*
*  BUGS
*		None
*
*  SEE ALSO
*     InitEffects()
*
********************************************************************************/
BOOL SeagullDraw( DDRAW_DATA *pDDrawData, int iMsg )
{
   SeagullCheckInputDevices();

   // Grab defense status from main app variable
   g_defense.byObjectAlive = nEffectsDefended;

   switch( iMsg )
   {
      case DRAW_EFFECT:
         SeagullCalculate();
         SeagullDrawBuffer( pDDrawData );      // Main drawing pipeline
         SeagullDrawScreen( pDDrawData );
         SeagullEraseBuffer( pDDrawData );
         break;

      case CALCULATE_AND_SKIP_FRAME:
         SeagullCalculate();
         break;

      default:
         break;   // Simply ignore future messages
   }

   return ( TRUE );
}


/******************************************************************************
*
*  NAME
*     SeagullCheckInputDevices
*
*  SYNOPSIS
*     SeagullCheckInputDevices()
*
*  DESCRIPTION
*     Checks for input from the mouse or keyboard.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     SeagullDraw(), SeagullCheckCheatCodes()
*
********************************************************************************/
static void SeagullCheckInputDevices( void )
{
   if ( LButtonPressed )
   {
      LButtonPressed = FALSE;
      SeagullStart();                        // Start a new seagull
   }

   // Ignore all other mouse button events
   if ( LButtonReleased )
      LButtonReleased = FALSE;
   if ( RButtonPressed )
      RButtonPressed = FALSE;
   if ( RButtonReleased )
      RButtonReleased = FALSE;

   // Check for cheat codes
   SeagullCheckCheatCodes();
}


/******************************************************************************
*
*  NAME
*     SeagullCheckCheatCodes
*
*  SYNOPSIS
*     SeagullCheckCheatCodes()
*
*  DESCRIPTION
*     Checks for cheat codes typed in from the keyboard.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     SeagullCheckInputDevices()
*
********************************************************************************/
static void SeagullCheckCheatCodes( void )
{   
   if ( ( szCode[ 0 ] == 'd' ) && !strcmp( szCode, "defense" ) )
   {
      nEffectsDefended ^= TRUE;
      if ( !nEffectsDefended )
         g_defense.fxObject.dwFlags = ERASE;
      ResetCode();
   }
}


/******************************************************************************
*
*  NAME
*     SeagullStart
*
*  SYNOPSIS
*     SeagullStart()
*
*  DESCRIPTION
*     Starts a new seagull which will drop its load onto the current mouse position.
*     Calculates the seagull's trajectory and bombing formation.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     None
*
*  SEE ALSO
*     SeagullCheckInputDevices(), SeagullDropPoop()
*
********************************************************************************/
static void SeagullStart( void )
{      
   seagull_t   *pSeagull, *pSeagullEnd;
   DWORD       dwSeagullCount;
   double      fAngle;           // Angle from center of screen to target
   double      fAnglePartition;  // Angle between center of screen and corner
                                 // This helps us determine whether the line
                                 // connecting the center of the screen and
                                 // the target will hit the x- or y-axis firsst.
	int         dx, dy;           // Represents the slope of the line from origin to target
	int         iStartX, iStartY; // Starting position of the seagull


   // Find room in object array to create a new seagull
   dwSeagullCount = 0;
   pSeagull       = &s_aSeagulls[ 0 ];
   pSeagullEnd    = &s_aSeagulls[ MAX_OBJECTS_SEAGULL ];
   while ( pSeagull < pSeagullEnd )
   {
      if ( pSeagull->dwState )
      {
         dwSeagullCount++;
         if ( dwSeagullCount == MAX_OBJECTS_SEAGULL )
            return;              // Can't start any more seagulls for now
      }
      else
      {
         break;                  // Found a free spot, start a new seagull
      }

      pSeagull++;
   }

   // Start the seagull's strafing sequence
   // Note that the actual poop will not be allocated until the seagull is
   // ready to drop its load.
   memset( pSeagull, 0, sizeof( *pSeagull ) );

   pSeagull->dwState                = SEAGULL_STATE_APPROACH;
   pSeagull->iTargetX               = nMouseX;              // Keep track of target's location
   pSeagull->iTargetY               = nMouseY;
   pSeagull->wNumFlaps              = ( rand() % ( MAX_FLAPS - MIN_FLAPS + 1 ) ) + MIN_FLAPS;   // Seagull flaps its wings this many times at the beginning
   pSeagull->dwTimeOfLastSquawk     = 0;

   pSeagull->fxSeagull.nFrame       = TEXTURE_SEAGULL_0;
   pSeagull->fxSeagull.nFrameTime   = 0;                    // Reset timer
   pSeagull->fxSeagull.nFlags       = 1;                    // Play anim forwards

   // Calculate orientation ( angle ).
   // An angle of 0 ( out of ( 2 * PI ) radians ) points to the right.
   fAngle                     = SeagullGetCursorAngle();
   pSeagull->fAngleStart      = ( float )( fAngle * 180.0 / PI );
   pSeagull->fAngleMax        = ( float )( ( rand() % ( TURNING_ANGLE_MAX - TURNING_ANGLE_MIN + 1 ) ) + TURNING_ANGLE_MIN );

   // Randomly choose where the seagull will turn after it drops its load
   pSeagull->fAngleDiff             = ( float )( ( ( rand() % ( ROTATE_MAX - ROTATE_MIN + 1 ) ) + ROTATE_MIN ) / 10 );
   if ( rand() & 1 )
      pSeagull->fAngleDiff *= -1.0f;   // Randomly choose between turning left or right

   // Calculate starting point of seagull's flight path
   // First, determine if the slope of the line is positive or negative
   dx = nMouseX - ( nScreenWidth >> 1 );
   dy = nMouseY - ( nScreenHeight >> 1 );

   // Make sure dx and dy are non-zero, so we don't try to divide by zero
   if ( !dx )
      dx = 1;
   else if ( !dy )
      dy = 1;

   // Partition unit circle into 4 quadrants
   // which indicate which edge of the screen the seagull will start from
   fAnglePartition = atan2( ( double ) ( nScreenHeight >> 1 ), ( double ) ( nScreenWidth >> 1 ) );

   if ( ( fAngle > ( PI - fAnglePartition ) ) || ( fAngle < -( PI - fAnglePartition ) ) )
   {
      // Seagull starts from the right edge of the screen
      iStartX = nScreenWidth + ( s_aTextures[ TEXTURE_SEAGULL_0 ].width >> 1 );
      iStartY = ( ( iStartX - ( nScreenWidth >> 1 ) ) * dy ) / dx + ( nScreenHeight >> 1 );
   }
   else if ( fAngle > fAnglePartition )
   {
      // Seagull starts from the bottom edge of the screen
      iStartY = nScreenHeight + ( s_aTextures[ TEXTURE_SEAGULL_0 ].height >> 1 );
      iStartX = ( ( ( iStartY - ( nScreenHeight >> 1 ) ) * dx ) / dy ) + ( nScreenWidth >> 1 );
   }
   else if ( fAngle < -fAnglePartition )
   {
      // Seagull starts from the top edge of the screen
      iStartY = -( s_aTextures[ TEXTURE_SEAGULL_0 ].height >> 1 );
      iStartX = ( ( ( iStartY - ( nScreenHeight >> 1 ) ) * dx ) / dy ) + ( nScreenWidth >> 1 );
   }
   else
   {
      // Seagull starts from the left edge of the screen
      iStartX = -( s_aTextures[ TEXTURE_SEAGULL_0 ].width >> 1 );
      iStartY = ( ( ( iStartX - ( nScreenWidth >> 1 ) ) * dy ) / dx ) + ( nScreenHeight >> 1 );
   }

   // Calculate velocity and magnitude of velocity
   pSeagull->fxSeagull.fAngle    = pSeagull->fAngleStart;
   pSeagull->fxSeagull.dwFlags   = ( DWORD )( ( nMouseX - iStartX ) / TIME_TO_HIT_TARGET );
   pSeagull->fxSeagull.dwFlags2  = ( DWORD )( ( nMouseY - iStartY ) / TIME_TO_HIT_TARGET );
   pSeagull->fMagnitude          = ( float ) sqrt( ( pSeagull->fxSeagull.dwFlags * pSeagull->fxSeagull.dwFlags ) + ( pSeagull->fxSeagull.dwFlags2 * pSeagull->fxSeagull.dwFlags2 ) );

   // Readjust starting position to account for rounding errors in the velocity calculation
   pSeagull->fxSeagull.x = nMouseX - ( s_aTextures[ TEXTURE_SEAGULL_0 ].width >> 1 ) - ( int )( pSeagull->fxSeagull.dwFlags ) * TIME_TO_HIT_TARGET;
   pSeagull->fxSeagull.y = nMouseY - ( s_aTextures[ TEXTURE_SEAGULL_0 ].height >> 1 ) - ( int )( pSeagull->fxSeagull.dwFlags2 ) * TIME_TO_HIT_TARGET;
}


/******************************************************************************
*
*  NAME
*     SeagullGetCursorAngle
*
*  SYNOPSIS
*     fAngle = SeagullGetCursorAngle()
*
*  DESCRIPTION
*     Gets the current angle in radians that the cursor makes with the
*     center of the screen, where 0 radians points straight to the right.
*     
*  INPUTS
*     None
*
*  OUTPUTS
*     double fAngle = Angle in radians.
*
*  BUGS
*     None
*
*  SEE ALSO
*     SeagullStart(), SeagullCalculate()
*
********************************************************************************/
static double SeagullGetCursorAngle( void )
{
   int      iStartX, iStartY;
   double   dx, dy;


   iStartX = nScreenWidth >> 1;
   iStartY = nScreenHeight >> 1;

   // Special case if mouse is at the exact center of the screen
   if ( ( nMouseX == iStartX ) && ( nMouseY == iStartY ) )
      iStartX = 0;

   // Calculate starting position
   dx = ( double )( nMouseX - iStartX );
   dy = ( double )( iStartY - nMouseY );   // flip y since y increases downward, not upward
   return ( atan2( dy, dx ) );             // Angle is in radians
}


/******************************************************************************
*
*  NAME
*     SeagullDropPoop
*
*  SYNOPSIS
*     SeagullDropPoop( pSeagull )
*
*  DESCRIPTION
*     Drops poop from the given seagull.
*     The poop inherits the seagull's velocity and direction.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     Assumes that there are three types of falling poop.
*
*  SEE ALSO
*     SeagullStart(), SeagullCalculate()
*
********************************************************************************/
static void SeagullDropPoop( seagull_t *pSeagull )
{
   FXOBJECT *pfxPoop;


   if ( !pSeagull )
      return;

   pSeagull->poopObject.pfxPoop = CreateFXObject();
   if ( !( pSeagull->poopObject.pfxPoop ) )
   {
      // Too much poop falling already, play no ammo sound
//      PlayWave( s_aiSounds[ SOUND_NO_AMMO ] );
      return;
   }

   pSeagull->poopObject.dwState = POOP_STATE_FALLING;

   pfxPoop = pSeagull->poopObject.pfxPoop;

   InitObject( FindEffect( EFFECT_NAME_SEAGULL ), pfxPoop,
               &s_aTextures[ TEXTURE_FALLING_POOP_A ],
               pSeagull->fxSeagull.x + ( s_aTextures[ TEXTURE_SEAGULL_0 ].width >> 1 ) - ( s_aTextures[ TEXTURE_FALLING_POOP_A ].width >> 1 ),
               pSeagull->fxSeagull.y + ( s_aTextures[ TEXTURE_SEAGULL_0 ].height >> 1 ) - ( s_aTextures[ TEXTURE_FALLING_POOP_A ].height >> 1 ) );
   pfxPoop->nFrame   = TEXTURE_FALLING_POOP_A + ( rand() % 3 );   // Assumes there are 3 types of falling poop

   // Poop inherits its velocity from the seagull
   pfxPoop->dwFlags  = pSeagull->fxSeagull.dwFlags ;  // x-velocity
   pfxPoop->dwFlags2 = pSeagull->fxSeagull.dwFlags2;  // y-velocity
   pfxPoop->fAngle   = pSeagull->fxSeagull.fAngle;    // Inherit seagull's angle
   pfxPoop->fScaleX  = ( float )( s_aTextures[ TEXTURE_FALLING_POOP_A ].width );
   pfxPoop->fScaleY  = ( float )( s_aTextures[ TEXTURE_FALLING_POOP_A ].height );

      // Calculate amount to scale down for each frame as the poop falls.
      // Poop will scale down to half of its original width and height.
   pfxPoop->nFlags   = ( int ) ( pfxPoop->fScaleX / ( float )( 2.0f * ( TIME_TO_HIT_TARGET - TIME_TO_DROP_POOP ) ) );

//   PlayWave( s_aiSounds[ SOUND_SQUAWK ] );
}


/******************************************************************************
*
*  NAME
*     SeagullCalculate
*
*  SYNOPSIS
*     SeagullCalculate()
*
*  DESCRIPTION
*     Calculates the position and appropriate frame to display for all
*     objects on the screen.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     None
*
*  SEE ALSO
*     SeagullDraw(), SeagullDefenseMove(), SeagullDefenseCheckForCollision()
*
********************************************************************************/
static void SeagullCalculate( void )
{
   seagull_t   *pSeagull, *pSeagullEnd;
   FXOBJECT    *pfxSeagull, *pfxPoop;


   // Update the cursor position
   g_fxCursor.x = nMouseX - ( s_aTextures[ TEXTURE_CURSOR_0 ].width >> 1 );
   g_fxCursor.y = nMouseY - ( s_aTextures[ TEXTURE_CURSOR_0 ].height >> 1 );

   // Update cursor frame
   g_fxCursor.nFrameTime++;
   if ( !( g_fxCursor.nFrameTime % ANIM_DELAY_CURSOR ) )
   {
      g_fxCursor.nFrame++;
      if ( g_fxCursor.nFrame > TEXTURE_CURSOR_5 )
         g_fxCursor.nFrame = TEXTURE_CURSOR_0;
   }

   // Update seagulls and falling poop
   pSeagull    = &s_aSeagulls[ 0 ];
   pSeagullEnd = &s_aSeagulls[ MAX_OBJECTS_SEAGULL ];
   while ( pSeagull < pSeagullEnd )
   {
      pfxSeagull = &( pSeagull->fxSeagull );
      switch( pSeagull->dwState )
      {
         case SEAGULL_STATE_INVALID:
            break;

         case SEAGULL_STATE_APPROACH:
         case SEAGULL_STATE_LEAVING:
            // Check to see if the seagull should squawk
            if ( ( GetTickCount() - pSeagull->dwTimeOfLastSquawk ) > SQUAWK_DELAY )
            {
               if ( ( rand() % 100 ) < SQUAWK_PROBABILITY )
               {
                  PlayWave( s_aiSounds[ SOUND_SQUAWK ] );
                  pSeagull->dwTimeOfLastSquawk = GetTickCount();
               }
            }
            pfxSeagull->nFrameTime++;

            // Check to see if it's time to drop the poop
            if ( pfxSeagull->nFrameTime == ( TIME_TO_DROP_POOP * ANIM_DELAY_SEAGULL_MOVE ) )
            {
               pSeagull->dwState = SEAGULL_STATE_LEAVING;
               SeagullDropPoop( pSeagull );

               // Make seagull flap its wings once more when it poops
               pSeagull->wNumFlaps++;
            }

            if ( !( pfxSeagull->nFrameTime % ANIM_DELAY_SEAGULL_MOVE ) )
            {
               if ( pSeagull->dwState == SEAGULL_STATE_LEAVING )
               {
                  float fAngleDiff;


                  fAngleDiff = pfxSeagull->fAngle - pSeagull->fAngleStart;
                  if ( pSeagull->fAngleDiff < 0.0f )
                     fAngleDiff *= -1.0f;

                  if ( fAngleDiff < pSeagull->fAngleMax )
                  {
                     // Rotate seagull and change velocity according to new vector
                     pfxSeagull->fAngle   += pSeagull->fAngleDiff;
                     pfxSeagull->dwFlags  = ( DWORD )( pSeagull->fMagnitude * cos( - pfxSeagull->fAngle * PI / 180.0f ) );
                     pfxSeagull->dwFlags2 = ( DWORD )( pSeagull->fMagnitude * sin( - pfxSeagull->fAngle * PI / 180.0f ) );
                  }
               }

               // Move the seagull
               pfxSeagull->x += ( int )( pfxSeagull->dwFlags );
               pfxSeagull->y += ( int )( pfxSeagull->dwFlags2 );
            }

            if ( ( pSeagull->wNumFlaps > 0 ) &&
                 !( pfxSeagull->nFrameTime % ANIM_DELAY_FLAPPING ) )
            {
               pfxSeagull->nFrame += pfxSeagull->nFlags; // Direction of animation
               if ( pfxSeagull->nFrame == TEXTURE_SEAGULL_0 )
               {
                  pSeagull->wNumFlaps--;
                  pfxSeagull->nFlags = 1;
               }
               else if ( pfxSeagull->nFrame == TEXTURE_SEAGULL_4 )
               {
                  pfxSeagull->nFlags = -1;
               }
            }
            break;

         default:
            break;
     }

     pfxPoop = pSeagull->poopObject.pfxPoop;
     switch( pSeagull->poopObject.dwState )
     {
         case POOP_STATE_INVALID:
            break;

         case POOP_STATE_FALLING:
            if ( pfxSeagull->nFrameTime == ( TIME_TO_HIT_TARGET * ANIM_DELAY_SEAGULL_MOVE ) )
            {
               pSeagull->poopObject.dwState = POOP_STATE_HIT_PAGE;
               PlayWave( s_aiSounds[ SOUND_HIT_PAGE ] );
               if ( pfxPoop->nFrame == TEXTURE_FALLING_POOP_A )
                  pfxPoop->nFrame = TEXTURE_SPLAT_A_0;
               else if ( pfxPoop->nFrame == TEXTURE_FALLING_POOP_B )
                  pfxPoop->nFrame = TEXTURE_SPLAT_B_0;
               else
                  pfxPoop->nFrame = TEXTURE_SPLAT_C_0;

               pfxPoop->x = pSeagull->iTargetX - ( s_aTextures[ TEXTURE_SPLAT_A_0 ].width >> 1 );
               pfxPoop->y = pSeagull->iTargetY - ( s_aTextures[ TEXTURE_SPLAT_A_0 ].height >> 1 );
            }

            // Scale and move the poop
            else if ( !( pfxSeagull->nFrameTime % ANIM_DELAY_SEAGULL_MOVE ) )
            {
               pfxPoop->fScaleX  -= ( float )( pfxPoop->nFlags );
               pfxPoop->fScaleY  -= ( float )( pfxPoop->nFlags );
               pfxPoop->x        += ( int )( pfxPoop->dwFlags ) + ( pfxPoop->nFlags >> 1 );     // Recenter scaled image
               pfxPoop->y        += ( int )( pfxPoop->dwFlags2 ) + ( pfxPoop->nFlags >> 1 );
            }
            break;

         case POOP_STATE_HIT_PAGE:
         case POOP_STATE_HIT_PAGE_END:
         case POOP_STATE_BLOCKED:
            break;

         default:
            break;
      }

      pSeagull++;
   }

   if ( g_defense.byObjectAlive )
   {
      SeagullDefenseCheckForCollision();
      SeagullDefenseMove();
   }
}


/******************************************************************************
*
*  NAME
*     SeagullDrawBuffer
*
*  SYNOPSIS
*     SeagullDrawBuffer( pDDrawData )
*
*  DESCRIPTION
*     Draws all objects into the offscreen buffer.
*
*  INPUTS
*     DDRAW_DATA *pDDrawData = Current DirectDraw device info.
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     SeagullDraw()
*
********************************************************************************/
static void SeagullDrawBuffer( DDRAW_DATA *pDDrawData )
{
   seagull_t   *pSeagull, *pSeagullEnd;
   FXOBJECT    *pfxPoop;


   // Draw defense
   if ( g_defense.byObjectAlive )
      DrawObjectToBuffer( pDDrawData, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ], 0 );

   // Draw poop first
   pSeagull     = &s_aSeagulls[ 0 ];
   pSeagullEnd  = &s_aSeagulls[ MAX_OBJECTS_SEAGULL ];
   while ( pSeagull < pSeagullEnd )
   {
      pfxPoop = pSeagull->poopObject.pfxPoop;
      switch( pSeagull->poopObject.dwState )
      {
         case POOP_STATE_INVALID:
            break;

         case POOP_STATE_FALLING:
            DrawObjectToBuffer( pDDrawData, pfxPoop, &s_aTextures[ pfxPoop->nFrame ], SCALE_IMAGE );
            break;

         case POOP_STATE_HIT_PAGE:
            DrawObjectToBuffer( pDDrawData, pfxPoop, &s_aTextures[ pfxPoop->nFrame ], ROTATE_IMAGE );
            break;

         case POOP_STATE_HIT_PAGE_END:
            DrawObjectToBuffer( pDDrawData, pfxPoop, &s_aTextures[ pfxPoop->nFrame ], ROTATE_IMAGE );
            DrawObjectToScreenCapture( pDDrawData, pfxPoop, &s_aTextures[ pfxPoop->nFrame ], ROTATE_IMAGE );
            break;

         case POOP_STATE_BLOCKED:
            break;

         default:
            break;
      }

      pSeagull++;
   }

   // Now draw seagulls
   pSeagull     = &s_aSeagulls[ 0 ];
   pSeagullEnd  = &s_aSeagulls[ MAX_OBJECTS_SEAGULL ];
   while ( pSeagull < pSeagullEnd )
   {
      switch( pSeagull->dwState )
      {
         case SEAGULL_STATE_INVALID:
            break;

         case SEAGULL_STATE_APPROACH:
         case SEAGULL_STATE_LEAVING:
            DrawObjectToBuffer( pDDrawData, &( pSeagull->fxSeagull ), &s_aTextures[ pSeagull->fxSeagull.nFrame ], ROTATE_IMAGE );
            break;

         default:
            break;
      }

      pSeagull++;
   }

   // Draw cursor
#if ROTATE_CURSOR
   g_fxCursor.fAngle = ( float ) ( SeagullGetCursorAngle() * 180.0 / PI );   // Convert radians to degrees
   DrawObjectToBuffer( pDDrawData, &g_fxCursor, &s_aTextures[ g_fxCursor.nFrame ], ROTATE_IMAGE );
#else
   DrawObjectToBuffer( pDDrawData, &g_fxCursor, &s_aTextures[ g_fxCursor.nFrame ], 0 );
#endif

   // Erase blocked objects after we're done drawing to the offscreen buffer
   pSeagull    = &s_aSeagulls[ 0 ];
   pSeagullEnd = &s_aSeagulls[ MAX_OBJECTS_SEAGULL ];
   while ( pSeagull < pSeagullEnd )
   {
      pfxPoop = pSeagull->poopObject.pfxPoop;
      if ( pSeagull->poopObject.dwState == POOP_STATE_BLOCKED )
            EraseObjectOnScreen( pDDrawData, pfxPoop, &s_aTextures[ pfxPoop->nFrame ] );

      pSeagull++;
   }
}


/******************************************************************************
*
*  NAME
*     SeagullDrawScreen
*
*  SYNOPSIS
*     SeagullDrawScreen( pDDrawData )
*
*  DESCRIPTION
*     Draws all objects from the offscreen buffer onto the front screen buffer.
*
*  INPUTS
*     DDRAW_DATA *pDDrawData = Current DirectDraw device info.
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     SeagullDraw()
*
********************************************************************************/
static void SeagullDrawScreen( DDRAW_DATA *pDDrawData )
{
   seagull_t   *pSeagull, *pSeagullEnd;
   FXOBJECT    *pfxSeagull, *pfxPoop;


   // Draw defense
   if ( g_defense.byObjectAlive )
      DrawBufferObjectToScreen( pDDrawData, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ] );

   // Draw seagulls
   pSeagull    = &s_aSeagulls[ 0 ];
   pSeagullEnd = &s_aSeagulls[ MAX_OBJECTS_SEAGULL ];
   while ( pSeagull < pSeagullEnd )
   {
      pfxSeagull = &( pSeagull->fxSeagull );
      switch( pSeagull->dwState )
      {
         case SEAGULL_STATE_INVALID:
            break;

         case SEAGULL_STATE_APPROACH:
         case SEAGULL_STATE_LEAVING:
            DrawBufferObjectToScreen( pDDrawData, pfxSeagull, &s_aTextures[ pfxSeagull->nFrame ] );
            break;

         default:
            break;
      }

      pfxPoop = pSeagull->poopObject.pfxPoop;
      switch( pSeagull->poopObject.dwState )
      {
         case POOP_STATE_INVALID:
            break;

         case POOP_STATE_FALLING:
         case POOP_STATE_HIT_PAGE:
         case POOP_STATE_HIT_PAGE_END:
            DrawBufferObjectToScreen( pDDrawData, pfxPoop, &s_aTextures[ pfxPoop->nFrame ] );
            break;

         case POOP_STATE_BLOCKED:
            break;

         default:
            break;
      }

      pSeagull++;
   }

   // Draw cursor
   DrawBufferObjectToScreen( pDDrawData, &g_fxCursor, &s_aTextures[ g_fxCursor.nFrame ] );

   // Erase the defense
   if ( g_defense.fxObject.dwFlags == ERASE )
   {
      EraseObjectInBuffer( pDDrawData, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ] );
      EraseObjectOnScreen( pDDrawData, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ] );
      g_defense.fxObject.dwFlags = 0;
      g_defense.fxObject.nFlags  = DEFENSE_STATE_SEEKING;   // Reset the defense mechanism
   }
}


/******************************************************************************
*
*  NAME
*     SeagullEraseBuffer
*
*  SYNOPSIS
*     SeagullEraseBuffer( pDDrawData )
*
*  DESCRIPTION
*     Erases all the objects that were previously drawn into the offscreen
*     buffer in order to restore the offscreen buffer back to its original state
*     so we can draw on a clean background again on the next iteration of
*     the drawing pipeline.
*
*     This also handles the ending state transitions, except for the defense mechanism.
*
*  INPUTS
*     DDRAW_DATA *pDDrawData = Current DirectDraw device info.
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     SeagullDraw()
*
********************************************************************************/
static void SeagullEraseBuffer( DDRAW_DATA *pDDrawData )
{
   seagull_t   *pSeagull, *pSeagullEnd;
   FXOBJECT    *pfxSeagull, *pfxPoop;


   // Erase seagulls and poop
   pSeagull    = &s_aSeagulls[ 0 ];
   pSeagullEnd = &s_aSeagulls[ MAX_OBJECTS_SEAGULL ];
   while ( pSeagull < pSeagullEnd )
   {
      pfxSeagull = &( pSeagull->fxSeagull );
      switch( pSeagull->dwState )
      {
         case SEAGULL_STATE_INVALID:
            break;

         case SEAGULL_STATE_APPROACH:
            EraseObjectInBuffer( pDDrawData, pfxSeagull, &s_aTextures[ pfxSeagull->nFrame ] );
            break;

         case SEAGULL_STATE_LEAVING:
            EraseObjectInBuffer( pDDrawData, pfxSeagull, &s_aTextures[ pfxSeagull->nFrame ] );

            // Kill the seagull if it's poop is done falling
            // and the seagull has left the screen
            if ( !( pSeagull->poopObject.dwState ) &&
                 ( ( pfxSeagull->x < -s_aTextures[ TEXTURE_SEAGULL_0 ].width ) ||
                   ( pfxSeagull->x > nScreenWidth ) ||
                   ( pfxSeagull->y < -s_aTextures[ TEXTURE_SEAGULL_0 ].height ) || 
                   ( pfxSeagull->y > nScreenHeight ) ) )
            {
               pSeagull->dwState = SEAGULL_STATE_INVALID;
            }
            break;

         default:
            break;
      }

      pfxPoop = pSeagull->poopObject.pfxPoop;
      switch ( pSeagull->poopObject.dwState )
      {
         case POOP_STATE_INVALID:
            break;

         case POOP_STATE_FALLING:
            EraseObjectInBuffer( pDDrawData, pfxPoop, &s_aTextures[ pfxPoop->nFrame ] );
            break;

         case POOP_STATE_HIT_PAGE:
            EraseObjectInBuffer( pDDrawData, pfxPoop, &s_aTextures[ pfxPoop->nFrame ] );
            if ( ( pfxPoop->nFrame == TEXTURE_SPLAT_A_3 ) || 
                 ( pfxPoop->nFrame == TEXTURE_SPLAT_B_3 ) ||
                 ( pfxPoop->nFrame == TEXTURE_SPLAT_C_3 ) )
            	pSeagull->poopObject.dwState = POOP_STATE_HIT_PAGE_END;
            else
               pfxPoop->nFrame++;
            break;

         case POOP_STATE_HIT_PAGE_END:
            EraseObjectInBuffer( pDDrawData, pfxPoop, &s_aTextures[ pfxPoop->nFrame ] );
            pfxPoop->nFlags               = 0;     // Clear drawing flags for automatically saved object
            pSeagull->poopObject.dwState  = POOP_STATE_INVALID;
            pSeagull->poopObject.pfxPoop  = NULL;  // Note that the global Projectile[] array will still contain the
                                                   // smeared poop from this seagull, which is what we want
            break;

         case POOP_STATE_BLOCKED:
            pfxPoop->x = -9999;  // Kluge, we move the object out of bounds so it won't be drawn.
            pfxPoop->y = -9999;  // The entry in the global Projectiles[] array will be erased later
                                 // when the effect is ended and the array is condensed
            pSeagull->poopObject.dwState  = POOP_STATE_INVALID;
            pSeagull->poopObject.pfxPoop  = NULL;
            break;

         default:
            break;
      }

      pSeagull++;
   }

   // Erase defense
   if ( g_defense.byObjectAlive )
      EraseObjectInBuffer( pDDrawData, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ] );

   // Erase cursor
   EraseObjectInBuffer( pDDrawData, &g_fxCursor, &s_aTextures[ g_fxCursor.nFrame ] );
}


/******************************************************************************
*
*  NAME
*		SeagullDefenseMove
*
*  SYNOPSIS
*     SeagullDefenseMove()
*
*  DESCRIPTION
*     Determines where the umbrella should move for the time being.
*     The umbrella will seek the first target scheduled to be bombed,
*     or it simply follows the cursor if there are no seagulls on screen.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     None
*
*  SEE ALSO
*     SeagullCalculate(), SeagullDefenseCheckForCollision()
*
********************************************************************************/
static void SeagullDefenseMove( void )
{
   seagull_t   *pSeagull, *pSeagullEnd;
   FXOBJECT    *pfxPoop;


   if ( g_defense.fxObject.nFlags == DEFENSE_STATE_SEEKING )
   {
      int         dx, dy;
      int         nFrameTarget;


      // Find a new target, which can be either the mouse cursor, 
      // or the first target scheduled to be hit.
      // The first target to be hit is simply the one with the
      // highest nFrameTime member.
      g_defense.nDestX  = DEFENSE_FOLLOW_CURSOR;   // Assume no grenades on screen for now
      nFrameTarget      = -1;
      pSeagull          = &s_aSeagulls[ 0 ];
      pSeagullEnd       = &s_aSeagulls[ MAX_OBJECTS_SEAGULL ];
      while ( pSeagull < pSeagullEnd )
      {
         pfxPoop = pSeagull->poopObject.pfxPoop;
         if ( ( pSeagull->dwState == SEAGULL_STATE_APPROACH ) ||
              ( pSeagull->poopObject.dwState == POOP_STATE_FALLING ) )
         {
            if ( pSeagull->fxSeagull.nFrameTime > nFrameTarget )
            {
               nFrameTarget      = pSeagull->fxSeagull.nFrameTime;
               g_defense.nDestX  = pSeagull->iTargetX;
               g_defense.nDestY  = pSeagull->iTargetY;
            }
         }

         pSeagull++;
      }

      if ( g_defense.nDestX == DEFENSE_FOLLOW_CURSOR )
      {
         g_defense.nDestX = nMouseX;
         g_defense.nDestY = nMouseY;
      }

      g_defense.nSpeed = ( rand() % ( DEFENSE_MAX_STEP - DEFENSE_MIN_STEP + 1 ) ) + DEFENSE_MIN_STEP;

      dx = g_defense.nDestX - ( g_defense.fxObject.x + DEFENSE_X1 + ( DEFENSE_X2 >> 1 ) );
      dy = g_defense.nDestY - ( g_defense.fxObject.y + DEFENSE_Y1 + ( DEFENSE_Y2 >> 1 ) );

      g_defense.fIncrX = ( float ) dx / ( float ) g_defense.nSpeed;
      g_defense.fIncrY = ( float ) dy / ( float ) g_defense.nSpeed;

      g_defense.nCount           = 0;      // This keeps track of which step we're on
      g_defense.fxObject.nFlags  = DEFENSE_STATE_MOVING;
   }

   // ----- Move the defense if applicable
   if ( g_defense.fxObject.nFlags == DEFENSE_STATE_MOVING )
   {
      g_defense.fxObject.x += ( int ) g_defense.fIncrX;
      g_defense.fxObject.y += ( int ) g_defense.fIncrY;

      g_defense.nCount++;                          // Increment the current step counter
      if ( g_defense.nCount > g_defense.nSpeed )   // We've reached our target
         g_defense.fxObject.nFlags = DEFENSE_STATE_SEEKING;
   }
}


/******************************************************************************
*
*  NAME
*		SeagullDefenseCheckForCollision
*
*  SYNOPSIS
*     SeagullDefenseCheckForCollision()
*
*  DESCRIPTION
*     Checks to see if the umbrella has blocked any falling poop.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     Assumes there are 3 different damage textures
*
*  SEE ALSO
*     SeagullCalculate(), SeagullDefenseMove()
*
********************************************************************************/
static void SeagullDefenseCheckForCollision( void )
{
   seagull_t   *pSeagull, *pSeagullEnd;
   TEXTURE     *pTextureUmbrella, *pTextureDamage;


   pTextureUmbrella  = &s_aTextures[ TEXTURE_DEFENSE ];
   pTextureDamage    = &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_0 + ( rand() % 3 ) ];   // Pick one of three damage textures

   // ----- Determine if any objects have just hit the umbrella
   pSeagull    = &s_aSeagulls[ 0 ];
   pSeagullEnd = &s_aSeagulls[ MAX_OBJECTS_SEAGULL ];
   while ( pSeagull < pSeagullEnd )
   {
      if ( pSeagull->poopObject.dwState == POOP_STATE_HIT_PAGE )
      {
         int         iDefenseX1, iDefenseX2, iDefenseY1, iDefenseY2;
         int         iWeaponX1, iWeaponX2, iWeaponY1, iWeaponY2;
         FXOBJECT *  pfxObject;


         pfxObject = pSeagull->poopObject.pfxPoop;

         // Check for collision using bounding boxes.
         iDefenseX1 = g_defense.fxObject.x + DEFENSE_X1;
         iDefenseX2 = g_defense.fxObject.x + DEFENSE_X2;
         iDefenseY1 = g_defense.fxObject.y + DEFENSE_Y1;
         iDefenseY2 = g_defense.fxObject.y + DEFENSE_Y2;

         iWeaponX1 = pfxObject->x + SEAGULL_COLLISION_BOX_X1;
         iWeaponX2 = pfxObject->x + SEAGULL_COLLISION_BOX_X2;
         iWeaponY1 = pfxObject->y + SEAGULL_COLLISION_BOX_Y1;
         iWeaponY2 = pfxObject->y + SEAGULL_COLLISION_BOX_Y2;

         if ( ( iDefenseX1 >= iWeaponX2 ) || ( iDefenseX2 <= iWeaponX1 ) ||
              ( iDefenseY1 >= iWeaponY2 ) || ( iDefenseY2 <= iWeaponY1 ) )
            goto next;        // Trivial reject, rectangles do not intersect at all

         // Seagull poop graphics are 128x64 but we're just using the first
         // frame of the animation sequence so the poop's effective width is 54 pixels, not 128.
    	   // Weapon collides with the defense!
         g_defense.ptDef.x = ( int )( pSeagull->iTargetX ) - ( 27 ) - g_defense.fxObject.x;
         g_defense.ptDef.y = ( int )( pSeagull->iTargetY ) - ( pTextureDamage->height >> 1 ) - g_defense.fxObject.y;

         if ( ( g_defense.ptDef.x >= DEFENSE_DRAW_X1 ) &&
              ( ( g_defense.ptDef.x + 54 ) <= DEFENSE_DRAW_X2 ) &&
              ( g_defense.ptDef.y >= DEFENSE_DRAW_Y1 ) &&
              ( ( g_defense.ptDef.y + pTextureDamage->height ) <= DEFENSE_DRAW_Y2 ) )
         {
            DrawTextureInTexture( pTextureUmbrella, pTextureDamage, &g_defense.ptDef );
         }

         pSeagull->poopObject.dwState  = POOP_STATE_BLOCKED;
         g_defense.fxObject.nFlags     = DEFENSE_STATE_SEEKING; // Seek new target
         PlayWave( s_aiSounds[ SOUND_BLOCKED ] );
      }
next:
      pSeagull++;
   }

}


/******************************************************************************
*
*  NAME
*		SeagullManualSave
*
*  SYNOPSIS
*     iSuccess = SeagullManualSave( pDDrawData, iMsg, pvData )
*
*  DESCRIPTION
*     Takes care of displaying the correct final splat image for the seagull effect.
*     Also loads and saves effect data in the same manner as the manual save
*     functions for the other weapon packs.
*
*  INPUTS
*     DDRAW_DATA *   pDDrawData  = Pointer to DirectDraw device.
*     int            iMsg        = Message to process, i.e. load, save, or display.
*     void *         pvData      = Data location for reads or writes.
*
*  OUTPUTS
*     int iSuccess = TRUE
*
*  BUGS
*     None
*
*  SEE ALSO
*
********************************************************************************/
int SeagullManualSave( DDRAW_DATA *pDDrawData, int iMsg, void *pvData )
{
   static   BOOL              s_bTexturesLoaded = FALSE;
   static   TEXTURE           s_aSaveTextures[ 3 ] = { { 0 }, { 0 }, { 0 } };
            TEXTURE *         pTexture;
            FXOBJECT *        pfxObject;
            URLDATA *         pURLData;
            seagullSaveData_t *pSaveData, *pSaveDataEnd;
            int               iEffectID;


   iEffectID = FindEffect( EFFECT_NAME_SEAGULL );

   switch( iMsg )
	{
	case MANUAL_DRAW_TO_BROWSER:
      if ( !s_bTexturesLoaded )
      {
         // Make sure these texture file names match with the LoadTexture() calls above
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanid04.pcx",   &s_aSaveTextures[ 0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanie04.pcx",   &s_aSaveTextures[ 1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Seagull\\7esplatanif04.pcx",   &s_aSaveTextures[ 2 ] );
         s_bTexturesLoaded = TRUE;
      }

		pfxObject   = ( FXOBJECT * ) pvData;
      if ( pfxObject->nFrame == TEXTURE_SPLAT_A_3 )
         pTexture = &s_aSaveTextures[ 0 ];
      else if ( pfxObject->nFrame == TEXTURE_SPLAT_B_3 )
         pTexture = &s_aSaveTextures[ 1 ];
      else if ( pfxObject->nFrame == TEXTURE_SPLAT_C_3 )
         pTexture = &s_aSaveTextures[ 2 ];
      else
         break;

      DrawObjectToBrowserBuffer( pDDrawData, pfxObject, pTexture, ROTATE_IMAGE );
		break;

   case MANUAL_LOADED_EFFECT_DATA:
      pURLData       = ( URLDATA * ) pvData;
      pSaveData      = ( seagullSaveData_t * )( pURLData->ptrData );
		pSaveDataEnd   = ( seagullSaveData_t * )( ( DWORD ) pSaveData + ( DWORD ) ( pURLData->nSize ) );

      while ( pSaveData < pSaveDataEnd )
      {
         pfxObject = CreateFXObject();
         if ( !pfxObject )
            break;

         InitObject( iEffectID, pfxObject, NULL, 0, 0 );
         pfxObject->x               = pSaveData->x;
         pfxObject->y               = pSaveData->y;
         pfxObject->nFrame          = pSaveData->iFrame;
         pfxObject->fAngle          = pSaveData->fAngle;
         pfxObject->nInsideBrowser  = TRUE;

         pSaveData++;
      }

      free( pURLData->ptrData ); // This is not very kosher!
      break;

	case MANUAL_SAVE_EFFECT_DATA:
      pURLData          = ( URLDATA * ) pvData;
      pURLData->nSize   = GetTotalFXObjectsForEffect( iEffectID ) * sizeof( seagullSaveData_t );
      pURLData->ptrData = malloc( pURLData->nSize );
      pSaveData         = ( seagullSaveData_t * )( pURLData->ptrData );
      pSaveDataEnd      = ( seagullSaveData_t * )( ( DWORD ) pSaveData + ( DWORD )( pURLData->nSize ) );
      if ( !pSaveData )
         break;                                 // Error! malloc() failed!

      pfxObject = GetNextFXObject( iEffectID, FIND_FIRST_OBJECT );
      if ( !pfxObject )
         break;

      pSaveData->x      = pfxObject->x;
      pSaveData->y      = pfxObject->y;
      pSaveData->iFrame = pfxObject->nFrame;
      pSaveData->fAngle = pfxObject->fAngle;
      pSaveData++;

      while ( pSaveData < pSaveDataEnd )
      {
         pfxObject = GetNextFXObject( iEffectID, FIND_NEXT_OBJECT );
         if ( !pfxObject )
            break;

     		pSaveData->x      = pfxObject->x;
     		pSaveData->y      = pfxObject->y;
     		pSaveData->iFrame = pfxObject->nFrame;
     		pSaveData->fAngle = pfxObject->fAngle;

         pSaveData++;
    	}
    	break;
	}

   return ( TRUE );
}



